'**************************************
'*  mocog v0.6                        *
'*  An emulation of the Motorola 6809 *
'*  (C) 2009 Michael Rychlik          *
'**************************************
'
'v0.0 - 11 May 2009 - Original version
'
'v0.1 - 28 May 2009 - First signs of working, NOP, ADDA $00, JMP $0000 !!!
'
'v0.2 - 15 Jun 2009 - Most arithmentic and logic ops working (No flag settings implemented yet)
'                     Immediate, Direct, Extended addressing modes working.
'
'v0.3 - 16 Jun 2009 - Added MUL.
'                     Got all indexed addresing modes working.
'
'v0.4 - 21 Jun 2009 - Too many changes to list here but:
'                   - Made MUL, PSHS, PSHU, PULS, PULU, TFR, EXG and DAA into overlays.
'                   - This now runs the first gcc ompiled "Hello world" program !
'
'v0.5 - 08 Jul 2009 - Implemented RTS, ABX, RTI, RESET, SWI, SWI2, SWI3
'                     Moved all conditional branch tests to overlays.
'                     Added the I/O op. OUT #portno. This is an "enhancement" to the 6809 instruction set ...
'                     ... the idea of which is to avoid having to check for an I/O address on every memory access.
'                     Changed all #defines to upper case.
'                     Added all Page II dispatch vectors (SWI3 and all the CMPU and CPMS ops).
'                     All op-code handlers are now in place. Only the correct flag setting need to be done.
'
'v0.6 - 14 Jul 2009   Fixed flags setting on COM and NEG.
'                     Fixed subroutine call ops as wrong PC value was being pushed.
'                     MoCog now passes all ex6809.asm addressing mode tests and COM, NEG !!!
'
CON
'==========================================================================================================
'THE FOLLOWING IS REQUIRED FOR THE HOMESPUN V024 COMPILER
'Uncomment one of the following CPU emulation types.

'Uncomment one of the following platform types.
'#define TRIBLADEPROP
'#define PROP_DEMO_BOARD

'Uncomment one or more feature enables.
'#define BANKED_MEMORY                                  'Uncomment if banked memory is to emulated.
'#define SINGLE_STEP                                    'Uncomment if "single step" emulation required.
'#define HEARTBEAT                                      'Uncomment if emulator is to flash a heartbeat LED
'==========================================================================================================


#ifdef TRIBLADEPROP
#elseifdef PROP_DEMO_BOARD
#else
#warn No HW platform type defined
#endif


  '6809 flag bits
  entire_flag     = %10000000
  fast_int_mask   = %01000000
  half_carry_flag = %00100000
  interrupt_mask  = %00010000
  negative_flag   = %00001000
  zero_flag       = %00000100
  overflow_flag   = %00000010
  carry_flag      = %00000001


  'I/O control block commands
  io_cmd_out   = $01
  io_cmd_in    = $02
  io_cmd_break = $03

#ifdef BANKED_MEMORY
  'Memory bank select port.
  bank_sel_port = $40
#endif


VAR
  long  cog


PUB start(cpu_params) : okay | reg_base, io_base

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
''   cpu_par_params is a pointer to CPU parameter list

  stop

  'Set up pointers to the register file
  reg_base := LONG[cpu_params]
  b_reg    := reg_base + 0
  a_reg    := reg_base + 1
  d_reg    := b_reg
  x_reg    := reg_base + 2
  y_reg    := reg_base + 4
  u_reg    := reg_base + 6
  s_reg    := reg_base + 8
  pc_reg   := reg_base + 10
  dp_reg   := reg_base + 13
  cc_reg   := reg_base + 12

  'Set up list of pointers to byte regs used by TFR and EXG ops
  a_reg_l  := a_reg
  b_reg_l  := b_reg
  cc_reg_l := cc_reg
  dp_reg_l := dp_reg


#ifdef TRIBLADEPROP
#elseifdef PROP_DEMO_BOARD
  ram_base    := LONG[cpu_params + 4]
  ram_size    := LONG[cpu_params + 8]
  rom_base    := LONG[cpu_params + 12]
  rom_address := LONG[cpu_params + 16]
#endif

  io_base    := LONG[cpu_params + 20]
  io_command := io_base
  io_port    := io_base + 4
  io_data    := io_base + 8

  dispatch_tab_addr     := @dispatch_table_0
  dispatch_tab_0_addr   := @dispatch_table_0
  dispatch_tab_1_addr   := @dispatch_table_1

  overlay_tab_addr       := @overlay_table

  index_postbyte_table_addr := @index_postbyte_table

  'The following code sets up the Overlay Load Parameters
  pshs_ovl_  := OverlayParams(@pshs_ovl,  @pshs_ovl_end)
  pshu_ovl_  := OverlayParams(@pshu_ovl,  @pshu_ovl_end)
  puls_ovl_  := OverlayParams(@puls_ovl,  @puls_ovl_end)
  pulu_ovl_  := OverlayParams(@pulu_ovl,  @pulu_ovl_end)
  mul_ovl_   := OverlayParams(@mul_ovl,   @mul_ovl_end)
  tfr_ovl_   := OverlayParams(@tfr_ovl,   @tfr_ovl_end)
  exg_ovl_   := OverlayParams(@exg_ovl,   @exg_ovl_end)
  daa_ovl_   := OverlayParams(@daa_ovl,   @daa_ovl_end)
  out_ovl_   := OverlayParams(@out_ovl,   @out_ovl_end)
  in_ovl_    := OverlayParams(@in_ovl,    @in_ovl_end)
  rti_ovl_   := OverlayParams(@rti_ovl,   @rti_ovl_end)
  reset_ovl_ := OverlayParams(@reset_ovl, @reset_ovl_end)
  abx_ovl_   := OverlayParams(@abx_ovl,   @abx_ovl_end)
  rts_ovl_   := OverlayParams(@rts_ovl,   @rts_ovl_end)
  swi_ovl_   := OverlayParams(@swi_ovl,   @swi_ovl_end)
  swi2_ovl_  := OverlayParams(@swi2_ovl,  @swi2_ovl_end)
  swi3_ovl_  := OverlayParams(@swi3_ovl,  @swi3_ovl_end)

  brn_ovl_   := OverlayParams(@brn_ovl,   @brn_ovl_end)
  bhi_ovl_   := OverlayParams(@bhi_ovl,   @bhi_ovl_end)
  bls_ovl_   := OverlayParams(@bls_ovl,   @bls_ovl_end)
  bhs_ovl_   := OverlayParams(@bhs_ovl,   @bhs_ovl_end)
  blo_ovl_   := OverlayParams(@blo_ovl,   @blo_ovl_end)
  bne_ovl_   := OverlayParams(@bne_ovl,   @bne_ovl_end)
  beq_ovl_   := OverlayParams(@beq_ovl,   @beq_ovl_end)
  bvc_ovl_   := OverlayParams(@bvc_ovl,   @bvc_ovl_end)
  bvs_ovl_   := OverlayParams(@bvs_ovl,   @bvs_ovl_end)
  bpl_ovl_   := OverlayParams(@bpl_ovl,   @bpl_ovl_end)
  bmi_ovl_   := OverlayParams(@bmi_ovl,   @bmi_ovl_end)
  bge_ovl_   := OverlayParams(@bge_ovl,   @bge_ovl_end)
  blt_ovl_   := OverlayParams(@blt_ovl,   @blt_ovl_end)
  bgt_ovl_   := OverlayParams(@bgt_ovl,   @bgt_ovl_end)
  ble_ovl_   := OverlayParams(@ble_ovl,   @ble_ovl_end)

  lbrn_ovl_  := OverlayParams(@lbrn_ovl,  @lbrn_ovl_end)
  lbhi_ovl_  := OverlayParams(@lbhi_ovl,  @lbhi_ovl_end)
  lbls_ovl_  := OverlayParams(@lbls_ovl,  @lbls_ovl_end)
  lbhs_ovl_  := OverlayParams(@lbhs_ovl,  @lbhs_ovl_end)
  lblo_ovl_  := OverlayParams(@lblo_ovl,  @lblo_ovl_end)
  lbne_ovl_  := OverlayParams(@lbne_ovl,  @lbne_ovl_end)
  lbeq_ovl_  := OverlayParams(@lbeq_ovl,  @lbeq_ovl_end)
  lbvc_ovl_  := OverlayParams(@lbvc_ovl,  @lbvc_ovl_end)
  lbvs_ovl_  := OverlayParams(@lbvs_ovl,  @lbvs_ovl_end)
  lbpl_ovl_  := OverlayParams(@lbpl_ovl,  @lbpl_ovl_end)
  lbmi_ovl_  := OverlayParams(@lbmi_ovl,  @lbmi_ovl_end)
  lbge_ovl_  := OverlayParams(@lbge_ovl,  @lbge_ovl_end)
  lblt_ovl_  := OverlayParams(@lblt_ovl,  @lblt_ovl_end)
  lbgt_ovl_  := OverlayParams(@lbgt_ovl,  @lbgt_ovl_end)
  lble_ovl_  := OverlayParams(@lble_ovl,  @lble_ovl_end)
  orcc_ovl_  := OverlayParams(@orcc_ovl,  @orcc_ovl_end)
  andcc_ovl_ := OverlayParams(@andcc_ovl, @andcc_ovl_end)
  sex_ovl_   := OverlayParams(@sex_ovl,   @sex_ovl_end)

  break_ovl_ := OverlayParams(@break_ovl, @break_ovl_end)
  '....... add more overlays as required.

  'Initilize registers
  pc := WORD[pc_reg]
  cc := BYTE[cc_reg]

  okay := cog := cognew(@entry, 0) + 1                     'Start emulator in a new COG

PUB OverlayParams (o_start, o_end) : params | len, hubend, cogend
'This code sets up the parameters for an overlay (in a format to increase overlay loading speed)
  len := o_end - o_start                                   'length of hub overlay (bytes)
  hubend := o_start + len - 1                              'hub END address (last long) +3 (speed quirk in overlay loader)
  cogend := ((@OVERLAY_START - @entry + len) / 4) - 1      'cog END address (last long)
  params := hubend << 16 + cogend                          'set parameters


PUB stop

'' Stop CPU simulator - frees a cog

  if cog
    cogstop(cog~ - 1)


DAT
'---------------------------------------------------------------------------------------------------------

                        org     0

'---------------------------------------------------------------------------------------------------------
entry
OVERLAY_LOAD_RET
                        jmp       #RESIDENT_CODE        'Go execute resident code first (loaded automatically at CogNew)
                                                        'Can use as a RET instruction AFTER execution of the loaded overlay
'---------------------------------------------------------------------------------------------------------
'RESIDENT ROUTINES follow...
'
RESIDENT_CODE
#ifdef TRIBLADEPROP
'the following are also used as variables later...
'The TriBladeProp Blade #2 driver presets the latch to enable SRAM -CE(U23)=0 and hold LE=0
datax                   mov     outa, ram_latched       ' -WE=1 -OE=1, all other bits=0
outx                    mov     dira, ram_dir_read      ' setup direction to read
#endif

#ifdef HEARTBEAT
                        mov     dira, heartbeat_pin     'Set Pins to output
#endif
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'Instruction fetch loop starts here
disable_interrupts
enable_interrupts
fetch
#ifdef SINGLE_STEP
'The following 2 lines do 6809 single stepping
                        mov     OVERLAY_PAR, break_ovl_    '\\ copy break_ovl's parameters
                        call    #OVERLAY_LOAD              '// load & execute break_ovl
#endif

#ifdef HEARTBEAT
                        xor     outa, heartbeat_pin        'N.B. If using heartbeat for KIPS measurement
                                                           'remember to multiply the frequency by 2.
#endif
                        mov     dispatch_tab_addr, dispatch_tab_0_addr

dispatch                call    #get_byte_immediate_f

dispatch_2              shl     data_8, #2                 'Mul by 4 to get a LONG offset into the table
                        add     data_8, dispatch_tab_addr  'Index into table
                        rdlong  vector, data_8             'Get vector from table
vect_1                  jmp     vector                     'Jump(indirect) to the instruction handler

vect_2
vect_3                  shr     vector, #9                 'shift to 2nd (or 3rd) vector
                        jmp     vector                     'Jump(indirect) to the instruction handler


pc                      long    0                          '6809's Pogram Conter
cc                      long    0                          '6809's flags

#ifdef HEARTBEAT
heartbeat_pin           long    %00000000_00000001_00000000_00000000
#endif

dispatch_tab_addr       long    0
dispatch_tab_0_addr     long    0-0
dispatch_tab_1_addr     long    0-0

vector                  long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Source Functions

get_byte_a              rdbyte  data_8, a_reg
                        jmp     #vect_2


get_byte_b              rdbyte  data_8, b_reg
                        jmp     #vect_2

get_byte_direct         call    #get_ea_direct_f
                        call    #read_memory_byte          'Get that byte operand
                        jmp     #vect_2


get_byte_indexed        call    #get_ea_indexed_f          'Get effective address from indexed address post byte into data_16
                        mov     address, effective_address     'FIXME Do wee need this ?
                        call    #read_memory_byte
                        jmp     #vect_2


get_byte_extended       call    #get_word_immediate_f
                        mov     address, data_16           'Use that as effective address of required byte
                        call    #read_memory_byte
                        jmp     #vect_2

get_word_extended       call    #get_word_immediate_f
                        mov     address, data_16           'Use that as effective address of required byte
                        call    #read_memory_word
                        jmp     #vect_2

get_byte_immediate      call    #get_byte_immediate_f
                        jmp     #vect_2


get_word_immediate      call    #get_word_immediate_f
                        jmp     #vect_2


get_word_direct         call    #get_ea_direct_f
                        call    #read_memory_word          'Get that WORD operand
                        jmp     #vect_2


get_word_indexed        call    #get_ea_indexed_f          'Get effective address from indexed address post byte into data_16
                        mov     address, effective_address  'FIXME Do wee need this
                        call    #read_memory_word
                        jmp     #vect_2

get_byte_immediate_f    mov     address, pc
                        call    #read_memory_byte          'Read the instruction BYTE there
                        add     pc, #1                     'Increment the program counter
get_byte_immediate_f_ret  ret

get_word_immediate_f    mov     address, pc
                        call    #read_memory_word          'Read the instruction WORD there
                        add     pc, #2                     'Increment the program counter
get_word_immediate_f_ret  ret



'---------------------------------------------------------------------------------------------------------


'---------------------------------------------------------------------------------------------------------
'Get Effective address functions
get_ea_indexed_f        call    #get_byte_immediate_f

                        mov     post_byte, data_8

                        mov     index_reg, data_8          'Extract required index reg from post byte bits 5 & 6
                        shr     index_reg, #4              'and make into an offet into the register file...
                        and     index_reg, #%00000110
                        add     index_reg, x_reg           '...and then a pointer to the required register.

                        test    post_byte, #%10000000 wz   'Post byte bit 7 indicates index reg + 5 bit offset addressing
              if_z      jmp     #index_plus_5_bit

                        and     data_8, #%00001111         'Extract the indexing mode nibble from the post byte
                        shl     data_8, #2                 'Mul by 4 to get long offset into table
                        add     data_8, index_postbyte_table_addr 'Look up in indexing dispatch table
                        rdlong  data_8, data_8             'Get vector from table
                        jmp     data_8                     'Jump(indirect) to the instruction handler

ea_done                 mov     address, effective_address    'FIXME Do we need this?

get_ea_indexed_f_ret    ret

index_plus_5_bit        and     data_8, #%00011111         'Extract 5 bit offset from post byte
                        shl     data_8, #27                'Offset is two's comp. so sign extend it.
                        sar     data_8, #27
                        rdword  effective_address, index_reg
                        add     effective_address, data_8  'Add it to the ea
                        jmp     #ea_done

'00 = X
'01 = Y
'10 = U
'11 = S

increment               LONG    0
effective_address       LONG    0
index_reg               LONG    0                          'Pointer reg used for indexed addressing.
post_byte               LONG    0
index_postbyte_table_addr LONG  0-0                        'Address of index postbyte dispatch table

check_indirection       test    post_byte, #$10 wz         'Check the indirection bit in the post_byte
              if_z      jmp     #ea_done
                        mov     address, effective_address 'Indirection required
                        call    #read_memory_word
                        mov     effective_address, data_16
                        jmp     #ea_done

',R+
index_post_inc_1        mov     increment, #1
index_post_inc          rdword  effective_address, index_reg
                        mov     address, effective_address
                        add     address, increment
                        wrword  address, index_reg
                        jmp     #check_indirection

',R++
index_post_inc_2        mov     increment, #2
                        jmp     #index_post_inc

',-R
index_pre_dec_1         mov     increment, #1
index_pre_dec           rdword  effective_address, index_reg
                        sub     effective_address, increment
                        wrword  effective_address, index_reg
                        jmp     #check_indirection

',--R
index_pre_dec_2         mov     increment, #2
                        jmp     #index_pre_dec

',R
index                   rdword  effective_address, index_reg
                        jmp     #check_indirection


'(+/- B),R
index_b                 rdbyte  data_8, b_reg              'Get the offset from B register
offset_8                rdword  effective_address, index_reg
offset_8_tail           shl     data_8, #24                'Offset is two's comp. so sign extend it.
                        sar     data_8, #24
                        add     effective_address, data_8  'Add the offset.
                        jmp     #check_indirection

'(+/- A),R
index_a                 rdbyte  data_8, a_reg              'Get the offset from A register
                        jmp     #offset_8


'(+/- 7 bit offset),R
index_offset_8          call    #get_byte_immediate_f
                        jmp     #offset_8

'(+/- 15 bit offset),R
index_offset_16         call    #get_word_immediate_f
offset_16               rdword  effective_address, index_reg
offset_16_tail          add     effective_address, data_16 'Add the offset.
                        jmp     #check_indirection

'(+/- D),R
index_d                 rdword  data_16, d_reg             'Get the offset register
                        jmp     #offset_16

'(+/- 7 bit offset),PC
pc_offset_8             call    #get_byte_immediate_f
                        mov     effective_address, pc
                        jmp     #offset_8_tail


'(+/- 15 bit offset),PC
pc_offset_16            call    #get_word_immediate_f
                        mov     effective_address, pc
                        jmp     #offset_16_tail

'[address]
extended_indirect       call    #get_word_immediate_f
                        mov     effective_address, data_16
                        jmp     #check_indirection         'FIXME Should check for lack of indirection here!!


get_ea_direct_f         'FIXME TEST THIS !!
                        call    #get_byte_immediate_f
                        rdbyte  effective_address, dp_reg
                        shl     effective_address, #8
                        add     effective_address, data_8
                        mov     address, effective_address  'FIXME Do we need this?
get_ea_direct_f_ret     ret

get_ea_direct           call    #get_ea_direct_f
                        jmp     #vect_2


get_ea_indexed          call    #get_ea_indexed_f          'Get effective address from indexed address post byte into data_16
                        jmp     #vect_2

get_ea_extended         call    #get_word_immediate_f
                        mov     effective_address, data_16
                        jmp     #vect_2
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Destination Functions

ld                      wrbyte  data_8, alu_reg
                        jmp     #fetch

ldd                     wrword  data_16, alu_reg wz
set_n_z_v0_double       andn    cc, #overflow_flag         'Clear overflow
set_n_z_double          muxz    cc, #zero_flag             'Set zero and negative flags
                        test    data_16, negative_word wc
                        muxc    cc, #negative_flag
                        jmp     #fetch

negative_word           long    $8000

st                      mov     data_8, alu
                        mov     address, effective_address
                        call    #write_memory_byte
                        jmp     #fetch

std                     mov     data_16, alu wz
                        mov     address, effective_address
                        call    #write_memory_word
                        jmp     #set_n_z_v0_double

lea                     wrword  effective_address, alu_reg
                        jmp     #fetch


put_byte_a              wrbyte  data_8, a_reg
                        jmp     #fetch

put_byte_b              wrbyte  data_8, b_reg
                        jmp     #fetch

                        jmp     #fetch

put_byte_ea             call    #write_memory_byte
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Memory Access Functions

#ifdef TRIBLADEPROP
'                                          +-------- -CS (U26 FLASH)   -+
'                                          |+------- -CS (J22 microSD)  +- Latched pins...
'                                          ||+------ -RST3              |    passes when LE=1
'             (P30) SO ---+                |||+----- -RST1              |    latches on  LE low going edge
'             (P31) SI --+|                ||||+---- -CE (U24 SRAM-hi)  |    latched on  LE=0
'                        ||+---- -WE(SDA)  |||||+--- -CE (U23 SRAM-lo)  |
'                        |||+--- -OE(SCL)  ||||||+-- A20                |
'                        ||||+--  LE       |||||||+- A19               -+
'                        |||||             ||||||||
'          P31...P0 --> %00xx0_xxxxxxxxxxx_xxxxxxxx_xxxxxxxx
ram_latched     long    %00110_00000000000_00000000_00000000 ' -WE=1 -OE=1 (LE=0)
ram_read        long    %00100_00000000000_00000000_00000000 ' -WE=1 -OE=0 (LE=0)
ram_write       long    %00010_00000000000_00000000_00000000 ' -WE=0 -OE=1 (LE=0)
ram_dir_read    long    %00110_11111111111_11111111_00000000 ' outputs WE, OE, A0-18, inputs D0-7
ram_dir_write   long    %00110_11111111111_11111111_11111111 ' outputs WE, OE, A0-18, D0-7
ram_dir_input   long    %00000_00000000000_00000000_00000000 ' all inputs (for Ram Disk access)
'                              +---------+ +------+ +------+
'                              A18......A8 A7....A0 D7....D0
'                                          Q7....Q0
'
'outx            long    0                                   'used as a temp address + WE/OE bits

'SRAM Initialisation: TRIBLADEPROP Blade #2 Driver presets latch to -CE(U23)

'Read a BYTE from location given by "address" of 6809 memory space into byte data_8
read_memory_byte
                        and     address, mask16k           'Restrict addresses to 16K
#ifdef BANKED_MEMORY
                        cmp     address, common_ram wc     'Address in resident RAM area ?
              if_c      add     address, bank_offset       'No: Add offset to current memory bank in ext RAM
#endif
                        mov     outx, address
                        shl     outx,#8                    'shift address 8 bits
                        or      outx, ram_read             'add -WE=1 -OE=0
                        mov     outa, outx                 'pc (shifted 8 bits + -WE=1 -OE=0)
                        nop                                'delay for sram access time
                        mov     data_8, ina                'read SRAM
                        and     data_8, #$FF               'extract 8 bits
read_memory_byte_ret    ret

write_memory_byte
#ifdef BANKED_MEMORY
                        and     address, mask_low_word     'Ensure address is within 64K
                        cmp     address, common_ram wc     'Address in resident RAM area ?
              if_c      add     address, bank_offset       'No: Add offset to current memory bank
#endif
              '''       cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              {if_b}    mov     outx, address
              {if_b}    shl     outx, #8                   'shift address 8 bits
              {if_b}    or      outx, ram_write            'add -WE=0 -OE=1
              {if_b}    mov     datax, data_8
              {if_b}    and     datax, #$FF                'ensure upper bytes=0
              {if_b}    or      outx, datax                'add data bits to be written
              {if_b}    mov     outa, outx                 'output -WE=0 -OE=1, address bits, data bits
              {if_b}    mov     dira, ram_dir_write        'set data bits to outputs
'             {if_b}    nop                                '25nS min delay (nothing reqd)
              {if_b}    or      outa, ram_latched          'set -WE=0->1 -OE=1->1  (end write)
              {if_b}    mov     dira, ram_dir_read         'set data bits to inputs
              {if_b}    mov     outa, ram_read             'set -WE=1 -OE=0
write_memory_byte_ret   ret

#ifdef BANKED_MEMORY
common_ram              long    $C000                      'Address of common (resident) RAM in 6809 space.
bank_offset             long    0                          'Offset of the current memory bank in external RAM
#endif

#elseifdef PROP_DEMO_BOARD

'Read a BYTE from location given by "address" of 6809 memory space into byte data_8
read_memory_byte        and     address, mask16k           'Restrict addresses to 16K
                        mov     hub_pointer, ram_base      'Point to the RAM
                        add     hub_pointer, address       'Add address offset
                        rdbyte  data_8, hub_pointer        'Read the byte from there
read_memory_byte_ret    ret


'Write the BYTE data_8 to location given by "address"
write_memory_byte       mov     hub_pointer, ram_base      'Point to the RAM
                        add     hub_pointer, address       'Add address offset
                        wrbyte  data_8, hub_pointer        'Write the byte there
write_memory_byte_ret   ret

'The following configuration "constants" set from COG PARameters.
ram_base                long    0-0                        'HUB address of the 6809 RAM area
ram_size                long    0-0                        'Length in bytes of available 6809 RAM
rom_base                long    0-0                        'HUB address of the BOOT ROM image
rom_start               long    0-0                        'ROM starts at this address in 6809 space
rom_address             long    0-0

hub_pointer             long    ram_base                   'Pointer into HUB RAM
mask16k                 long    $3FFF
#endif

'Read a WORD from location given by "address" of 6809 memory space into byte data_16
read_memory_word        call    #read_memory_byte          'Read the high byte
                        mov     data_16, data_8
                        shl     data_16, #8
                        add     address, #1
                        call    #read_memory_byte          'Read the low byte
                        or      data_16, data_8
read_memory_word_ret    ret

'Write the WORD data_16 to location given by "address"
write_memory_word       mov     data_8, data_16
                        ror     data_8, #8
                        call    #write_memory_byte         'Write the HIGH byte
                        add     address, #1
                        rol     data_8, #8                 'Write the LOW byte
                        call    #write_memory_byte
write_memory_word_ret   ret


'6809 RAM read/write vars.
address                 long    0                          'Address param for read/write funcs
data_8                  long    0                          'Data bytes read into here
data_16                 long    0                          'Data words read into here
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'6809 Operation Functions
select_reg_a            mov     alu_reg, a_reg
read_alu_byte           rdbyte  alu, alu_reg
                        jmp     #vect_3

select_reg_b            mov     alu_reg, b_reg
                        jmp     #read_alu_byte

select_reg_d            mov     alu_reg, d_reg
read_alu_word           rdword  alu, alu_reg
                        jmp     #vect_3

select_reg_x            mov     alu_reg, x_reg
                        jmp     #read_alu_word

select_reg_y            mov     alu_reg, y_reg
                        jmp     #read_alu_word

select_reg_u            mov     alu_reg, u_reg
                        jmp     #read_alu_word

select_reg_s            mov     alu_reg, s_reg
                        jmp     #read_alu_word


alu                     LONG    0                          'Arithmetic Logic Unit accumulator
flags                   LONG    0                          'The 6809 flag register working copy

neg_byte                cmp     data_8, #$00 wz            'Check for carry
                        muxnz   cc, #carry_flag
                        cmp     data_8, #$80 wz            'Check for overlow
                        muxz    cc, #overflow_flag
                        neg     data_8, data_8 wz          'Do the NEG
                        muxz    cc, #zero_flag
                        test    data_8, #%10000000 wc      'Check for negative result
                        muxc    cc, #negative_flag


                        jmp     #vect_3

com_byte                xor     data_8, #$FF wz
                        andn    cc, #overflow_flag         'V always cleared
                        or      cc, #carry_flag
                        jmp     #set_zn                    'FIXME set_zn, set_n_z !!!!!


lsr_byte                shr     data_8, #1 wc
                        muxc    cc, #carry_flag
                        jmp     #vect_3


ror_byte                test    cc, #carry_flag wz         'Set operands bit 8 from carry flag
                        muxnz   data_8, #%100000000
                        shr     data_8, #1 wc
                        muxc    cc, #carry_flag
                        jmp     #vect_3

lsl_byte                shl     data_8, #1
                        test    data_8, #%100000000 wz
                        muxnz   cc, #carry_flag
                        jmp     #vect_3

rol_byte                shl     data_8, #1
                        test    cc, #carry_flag wz         'Set bit 1 from carry flag
                        muxnz   data_8, #%00000001
                        test    data_8, #%100000000 wz
                        muxnz   cc, #carry_flag
                        jmp     #vect_3

asr_byte                shl     data_8, #24
                        sar     data_8, #25 wc
                        muxc    cc, #carry_flag
                        jmp     #vect_3

dec_byte                sub     data_8, #1
                        jmp     #vect_3

inc_byte                add     data_8, #1
set_zn                  muxz    cc, #zero_flag
                        test    data_8, #%10000000 wz
                        muxnz   cc, #negative_flag
                        jmp     #vect_3

tst_byte                sub     data_8, #0 wz
                        muxz    cc, #zero_flag
                        test    data_8, #%10000000 wz
                        muxnz   cc, #negative_flag
                        jmp     #fetch

clr_byte                mov     data_8, #0
                        andn    cc, #(negative_flag | overflow_flag | carry_flag)
                        or      cc, #zero_flag
                        jmp     #vect_3


sub_                    sub     alu, data_8                'HNZVC
                        wrbyte  alu, alu_reg
                        jmp     #fetch

cmp_                    cmp     alu, data_8 wz             'HNZVC
                        jmp     #set_n_z

cmpd                    rdword  alu, alu_reg               'NZVC
                        sub     alu, data_16 wz
                        jmp     #set_n_z_double

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'
sbc                     test    flags, #carry_flag wc      'Set Props carry from 6809 carry flag
                        subx    alu, data_8 wz             'HNZVC Sub with carry

write_alu_byte          wrbyte  alu, alu_reg               'Write out the result reg.
                        jmp     #set_n_z                   '


set_n_z_v0              andn    cc, #overflow_flag         'Clear overflow

set_n_z                 muxz    cc, #zero_flag             'Set zero and negative flags
                        test    alu, %10000000 wc
                        muxc    cc, #negative_flag
                        jmp     #fetch

and_                    and     alu, data_8 wz
                        jmp     #write_alu_byte

bit                     test    alu, data_8 wz
                        jmp     #set_n_z_v0


eor                     xor     alu, data_8 wz
                        jmp     #write_alu_byte


adc                     test    flags, #carry_flag wc      'Set Props carry from 6809 carry flag
                        addx    alu, data_8 wz             'HNZVC   Add with carry
                        jmp     #write_alu_byte

or_                     or      alu, data_8 wz
                        jmp     #write_alu_byte


add_                    add     alu, data_8                'HNZVC
                        jmp     #write_alu_byte


addd                    add     alu, data_16               'NZVC
write_alu_word          wrword  alu, alu_reg
                        jmp     #fetch

subd                    sub     alu, data_16               'NZVC
                        jmp     #write_alu_word


'---------------------------------------------------------------------------------------------------------

'Push a WORD onto the stack at address from data_8 (Yes data_8!!) decrement address by 2
push_word               sub     address, #1
                        call    #write_memory_byte
                        sub     address, #1
                        shr     data_8, #8
                        call    #write_memory_byte
push_word_ret           ret

push_pc                 rdword  address, s_reg             'Get current stack pointer
                        mov     data_8, pc
                        call    #push_word
                        wrword  address, s_reg
push_pc_ret             ret

'Push a BYTE onto the stack at address from data_8
push_byte               sub     address, #1
                        call    #write_memory_byte
push_byte_ret           ret


'Pop a WORD from the stack at address to data_16
pop_word                call    #read_memory_byte          'Read the high byte
                        add     address, #1
                        mov     data_16, data_8
                        shl     data_16, #8
                        call    #read_memory_byte          'Read the low byte
                        add     address, #1
                        or      data_16, data_8
pop_word_ret            ret

'Pop a BYTE from the stack at address to data_8
pop_byte                call    #read_memory_byte
                        add     address, #1
pop_byte_ret            ret
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
not_implemented
illegal
halt                    mov     OVERLAY_PAR, break_ovl_    '\\ copy break_ovl's parameters
                        call    #OVERLAY_LOAD              '// load & execute break_ovl
                        jmp     #fetch

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Branch functions
jmp_                    mov     pc, effective_address
                        jmp     #fetch


branch_conditional      test    alu, #1 wz
              if_nz     jmp     #branch
                        add     pc, #1
                        jmp     #fetch


branch                  call    #get_byte_immediate_f
                        shl     data_8, #24                'Offset is two's comp. so sign extend it.
                        sar     data_8, #24
                        add     pc, data_8
                        jmp     #fetch


bsr                     call    #get_byte_immediate_f
                        mov     data_16, data_8            'FIXME: This data_8/16 thing is nuts!
                        shl     data_16, #24               'Offset is two's comp. so sign extend it.
                        sar     data_16, #24
                        call    #push_pc
                        add     pc, data_16
                        jmp     #fetch


lbsr                    call    #get_word_immediate_f
                        call    #push_pc
                        add     pc, data_16
                        jmp     #fetch

jsr                     call    #push_pc
                        mov     pc, effective_address
                        jmp     #fetch


lbranch_conditional     test    alu, #1 wz
              if_nz     jmp     #lbranch
                        add     pc, #2
                        jmp     #fetch

lbranch                 call    #get_word_immediate_f
                        add     pc, data_16
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------
'Handle prefixal op codes.
page1                   mov     dispatch_tab_addr, dispatch_tab_1_addr  'Page 1 ops
                        jmp     #dispatch

                                                           'N.B. See dispatch_table_1 to see how this works.
page2                   call    #get_byte_immediate_f      'Get the Page II op-code byte
                        sub     data_8, #1                 'Adjust it to correct entry in Page I dispatch table
                        and     data_8, #$FF                'Ensure in range table length
                        jmp     #dispatch_2                'Do it.
'---------------------------------------------------------------------------------------------------------
'Load and execute an ovelay accoring to overlay number in dispatch table
vector_overlay          mov     ovl, vector                'Extract overlay param number from vector bits 2 to 26
                        shr     ovl, #7
                        and     ovl, #%11111100
                        add     ovl, overlay_tab_addr      'Offset into overlay parameter table
                        rdlong  OVERLAY_PAR, ovl           'Get this overlays parameters
                        jmp     #OVERLAY_LOAD              'load it and run it.

ovl                     long      0
overlay_tab_addr        long      0-0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Pointers to 6809 registers in HUB RAM
'Do not change the order of these or inexed addressing will break.
a_reg                   long    0-0
b_reg                   long    0-0
d_reg                   long    0-0
x_reg                   long    0-0
y_reg                   long    0-0
u_reg                   long    0-0
s_reg                   long    0-0
pc_reg                  long    0-0
dp_reg                  long    0-0
cc_reg                  long    0-0

alu_reg                 long    0                          'Can point to any register that
                                                           'is accumulator at the time
'List of byte registers used by EXG and TFR
'Do not change the order of these
byte_regs_list
a_reg_l                 long    0-0
b_reg_l                 long    0-0
cc_reg_l                long    0-0
dp_reg_l                long    0-0
'---------------------------------------------------------------------------------------------------------
'The I/O control block
'Do not change the order of these
io_command              long    0-0
io_port                 long    0-0
io_data                 long    0-0
'---------------------------------------------------------------------------------------------------------

#warn There 45 reserved and unused longs in COG here
#warn Reserved for use with TriBlade + banked memory platform
'long 0[45]
'---------------------------------------------------------------------------------------------------------
' OVERLAY LOADER follows...
' An even number of longs will be loaded (for efficiency)
' Flags c & z will be maintained
'
Overlay_par             long    0-0                        'overlay parameters for the OVERLAY_LOAD

break_ovl_              long    0-0

_0x400                  long    $0000_0400                 'inc/decrement destination by 2
_djnz0                  djnz    overlay_par, #overlay_copy2 'prototype instruction (moved to overlay instruction)
t1                      long    0                          'used to determine if overlay already loaded (can be used by other code)

OVERLAY_LOAD            mov     OVERLAY_START, _djnz0      'Copy djnz instruction to head of overlay area.
                        movd    overlay_copy2, overlay_par 'move cog END address into rdlong instruction
                        sub     overlay_par, #1            'decrement cog End address by 1
                        movd    overlay_copy1, overlay_par 'move cog END-1 address into rdlong instruction
                        shr     overlay_par, #16           'extract the overlay## hub END address (remove cog address)
overlay_copy2           rdlong  0-0, overlay_par           'copy long from hub to cog   (hptr ignores last 2 bits!)
                        sub     overlay_par, #7            'decrement hub ptr by 1 long (prev by 1, now by 7)
                        sub     overlay_copy2, _0x400      'decrement cog (destination) address by 2
overlay_copy1           rdlong  0-0, overlay_par           'copy long from hub to cog
                        sub     overlay_copy1, _0x400      'decrement cog (destination) address by 2
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
OVERLAY_START '<--- djnz  overlay_par,#overlay_copy2    'decrement hub ptr by 1 long (now by 1, next by 7)
              '^^^ The above instruction is moved in by the loader and is overwritten by the first overlay
              '    instruction during the final overlay load (rdlong) instruction. (loading is done in reverse)
'---------------------------------------------------------------------------------------------------------
                        fit       $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'PSHS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
pshs_ovl                call    #get_byte_immediate_f
                        mov     post_byte, data_8
                        rdword  address, s_reg             'Get current stack pointer

                        test    post_byte, #%10000000 wz
              if_nz     mov     data_8, pc
              if_nz     call    #push_word

                        test    post_byte, #%01000000 wz
              if_nz     rdword  data_8, u_reg
              if_nz     call    #push_word

                        test    post_byte, #%00100000 wz
              if_nz     rdword  data_8, y_reg
              if_nz     call    #push_word

                        test    post_byte, #%00010000 wz
              if_nz     rdword  data_8, x_reg
              if_nz     call    #push_word

                        test    post_byte, #%00001000 wz
              if_nz     rdbyte  data_8, dp_reg
              if_nz     call    #push_byte

                        test    post_byte, #%00000100 wz
              if_nz     rdbyte  data_8, b_reg
              if_nz     call    #push_byte

                        test    post_byte, #%00000010 wz
              if_nz     rdbyte  data_8, a_reg
              if_nz     call    #push_byte

                        test    post_byte, #%00000001 wz
              if_nz     mov     data_8, cc
              if_nz     call    #push_byte

                        wrword  address, s_reg
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
pshs_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'PSHU overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
pshu_ovl                call    #get_byte_immediate_f
                        mov     post_byte, data_8
                        rdword  address, u_reg             'Get current stack pointer

                        test    post_byte, #%10000000 wz   'FIXME Doing all these pushes in loops
              if_nz     mov     data_8, pc                 'would save space and overlay load time!
              if_nz     call    #push_word

                        test    post_byte, #%01000000 wz
              if_nz     rdword  data_8, u_reg
              if_nz     call    #push_word

                        test    post_byte, #%00100000 wz
              if_nz     rdword  data_8, y_reg
              if_nz     call    #push_word

                        test    post_byte, #%00010000 wz
              if_nz     rdword  data_8, x_reg
              if_nz     call    #push_word

                        test    post_byte, #%00001000 wz
              if_nz     rdbyte  data_8, dp_reg
              if_nz     call    #push_byte

                        test    post_byte, #%00000100 wz
              if_nz     rdbyte  data_8, b_reg
              if_nz     call    #push_byte

                        test    post_byte, #%00000010 wz
              if_nz     rdbyte  data_8, a_reg
              if_nz     call    #push_byte

                        test    post_byte, #%00000001 wz
              if_nz     mov     data_8, cc
              if_nz     call    #push_byte

                        wrword  address, u_reg
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
pshu_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'PULS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
puls_ovl                call    #get_byte_immediate_f
                        mov     post_byte, data_8
                        rdword  address, s_reg             'Get current stack pointer

                        test    post_byte, #%00000001 wz
              if_nz     call    #pop_byte
              if_nz     mov     cc, data_8

                        test    post_byte, #%00000010 wz
              if_nz     call    #pop_byte
              if_nz     wrbyte  data_8, a_reg

                        test    post_byte, #%00000100 wz
              if_nz     call    #pop_byte
              if_nz     wrbyte  data_8, b_reg

                        test    post_byte, #%00001000 wz
              if_nz     call    #pop_byte
              if_nz     wrbyte  data_8, dp_reg

                        test    post_byte, #%00010000 wz
              if_nz     call    #pop_word
              if_nz     wrword  data_16, x_reg

                        test    post_byte, #%00100000 wz
              if_nz     call    #pop_word
              if_nz     wrword  data_16, y_reg

                        test    post_byte, #%01000000 wz
              if_nz     call    #pop_word
              if_nz     wrword  data_16, u_reg

                        test    post_byte, #%10000000 wz
              if_nz     call    #pop_word
              if_nz     mov     pc, data_16

:skip_pc                wrword  address, s_reg
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
puls_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'PULU overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
pulu_ovl                call    #get_byte_immediate_f
                        mov     post_byte, data_8
                        rdword  address, u_reg             'Get current stack pointer

                        test    post_byte, #%00000001 wz
              if_nz     call    #pop_byte
              if_nz     mov     cc, data_8

                        test    post_byte, #%00000010 wz
              if_nz     call    #pop_byte
              if_nz     wrbyte  data_8, a_reg

                        test    post_byte, #%00000100 wz
              if_nz     call    #pop_byte
              if_nz     wrbyte  data_8, b_reg

                        test    post_byte, #%00001000 wz
              if_nz     call    #pop_byte
              if_nz     wrbyte  data_8, dp_reg

                        test    post_byte, #%00010000 wz
              if_nz     call    #pop_word
              if_nz     wrword  data_8, x_reg

                        test    post_byte, #%00100000 wz
              if_nz     call    #pop_word
              if_nz     wrword  data_8, y_reg

                        test    post_byte, #%01000000 wz
              if_nz     call    #pop_word
              if_nz     wrword  data_8, u_reg

                        test    post_byte, #%10000000 wz
              if_nz     call    #pop_word
              if_nz     mov     pc, data_8

:skip_pc                wrword  address, u_reg
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
pulu_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'MUL overlay. This is a "load/execute" overlay.
                        org OVERLAY_START
mul_ovl                 rdword  alu, d_reg
                        mov     data_16, alu
                        andn    data_16, #$FF
                        shl     data_16, #15
#ifdef LOOP_MUL
#warn Using loop multiply
                        mov     data_8, #8

:loop                   shr     alu, #1 wc
              if_c      add     alu, data_16
                        djnz    data_8, #:loop
#else
#warn Using unrolled multiply
                        shr     alu, #1 wc
              if_c      add     alu, data_16
                        shr     alu, #1 wc
              if_c      add     alu, data_16
                        shr     alu, #1 wc
              if_c      add     alu, data_16
                        shr     alu, #1 wc
              if_c      add     alu, data_16
                        shr     alu, #1 wc
              if_c      add     alu, data_16
                        shr     alu, #1 wc
              if_c      add     alu, data_16
                        shr     alu, #1 wc
              if_c      add     alu, data_16
                        shr     alu, #1 wc
              if_c      add     alu, data_16
#endif
                        shr     alu, #16 wz
              if_z      or      cc, #zero_flag
                        test    alu, #%10000000 wz
                        muxnz   cc, #carry_flag
                        wrword  alu, d_reg
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
mul_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'TFR overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
tfr_ovl                 call    #get_byte_immediate_f
                        test    data_8, #%10001000 wc, wz  'Check for same size src and dest regs.
              if_c      jmp     #illegal

                        mov     data_16, data_8
                        and     data_8, #%00000111         'Extract dest reg from post byte bits 3..0

                        shr     data_16, #4                'Extract src reg from post byte bits 7..4
                        and     data_16, #%00000111

              if_nz     jmp     #:tfr_8

:tfr_16                 'FIXME Check source and dest for greater than 5 !!
                        movs    :rdsrc_16, #d_reg          'Set the source field of the "read src" instruction
                        add     :rdsrc_16, data_16

                        movs    :wrdst_16, #d_reg          'Set the source field of the "write dst" instruction
                        add     :wrdst_16, data_8

:rdsrc_16               rdword  alu, 0-0                   'Do the reg to reg tranfer.
:wrdst_16               wrword  alu, 0-0

                        cmp     data_8, #5 wz              'Check if the PC was updated in HUB
              if_z      mov     pc, alu                    'If so up date in COG also
                        jmp     #fetch

:tfr_8                  'FIXME Check source and dest for greater than 3 !!
                        movs    :rdsrc_8, #byte_regs_list  'Set the source field of the "read src" instruction
                        add     :rdsrc_8, data_16

                        movs    :wrdst_8, #byte_regs_list  'Set the source field of the "write dst" instruction
                        add     :wrdst_8, data_8

:rdsrc_8                rdbyte  alu, 0-0                   'Do the reg to reg tranfer.
:wrdst_8                wrbyte  alu, 0-0

                        cmp     data_8, #2 wz              'Check if CC was updated in HUB
              if_z      mov     cc, alu                    'If so up date in COG also
                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
tfr_ovl_end
                        fit     $1F0

'TFR and EXG post byte bitfields.
'0000 =	A:B	   	1000 =	A
'0001 =	X		1001 =	B
'0010 =	Y		1010 =	CCR
'0011 =	US		1011 =	DPR
'0100 =	SP		1100 =	Undefined
'0101 =	PC		1101 =	Undefined
'0110 =	Undefined	1110 =	Undefined
'0111 =	Undefined	1111 =	Undefined
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'EXG overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
exg_ovl                 call    #get_byte_immediate_f
                        test    data_8, #%10001000 wc, wz  'Check for same size src and dest regs.
              if_c      jmp     #illegal

                        mov     data_16, data_8
                        and     data_8, #%00000111         'Extract dest reg from post byte bits 3..0

                        shr     data_16, #4                'Extract src reg from post byte bits 7..4
                        and     data_16, #%00000111

              if_nz     jmp     #:tfr_8

:tfr_16                 'FIXME Check source and dest for greater than 5 !!
                        add     data_16, #d_reg            'Set the source field of...
                        movs    :rdsrc_16, data_16         '...the "read src" instruction...
                        movs    :wrsrc_16, data_16         '...and "write src" instruction.

                        add     data_8, #d_reg             'Set the source field of...
                        movs    :rddst_16, data_8          '...the "read dst" instruction...
                        movs    :wrdst_16, data_8          '...and "write dst" instruction.

:rdsrc_16               rdword  alu, 0-0                   'Do the reg to reg exchange.
:rddst_16               rdword  address, 0-0
:wrsrc_16               wrword  address, 0-0
:wrdst_16               wrword  alu, 0-0

                        cmp     data_8, #pc_reg wz         'Check if the PC was updated in HUB
              if_z      mov     pc, alu                    'If so up date in COG also
                        cmp     data_16, #pc_reg wz        'Check if the PC was updated in HUB
              if_z      mov     pc, address                'If so up date in COG also
                        jmp     #fetch

:tfr_8                  'FIXME Check source and dest for greater than 5 !!
                        add     data_16, #byte_regs_list   'Set the source field of...
                        movs    :rdsrc_8, data_16          '...the "read src" instruction...
                        movs    :wrsrc_8, data_16          '...and "write src" instruction.

                        add     data_8, #byte_regs_list    'Set the source field of...
                        movs    :rddst_8, data_8           '...the "read dst" instruction...
                        movs    :wrdst_8, data_8           '...and "write dst" instruction.

:rdsrc_8                rdbyte  alu, 0-0                   'Do the reg to reg exchange.
:rddst_8                rdbyte  address, 0-0
:wrsrc_8                wrbyte  address, 0-0
:wrdst_8                wrbyte  alu, 0-0

                        cmp     data_8, #cc_reg_l wz       'Check if CC was updated in HUB
              if_z      mov     cc, alu                    'If so up date in COG also
                        cmp     data_16, #cc_reg_l wz      'Check if CC was updated in HUB
              if_z      mov     cc, address                'If so up date in COG also
                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
exg_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'DAA overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
daa_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
daa_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'RTI overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
rti_ovl                 call    #pop_byte
                        mov     cc, data_8
                        test    cc, #entire_flag wz
              if_z      jmp     #:pc_only
                        call    #pop_byte
                        wrbyte  data_8, a_reg   'FIXME This could probably be pop_word into d_reg
                        call    #pop_byte
                        wrbyte  data_8, b_reg
                        call    #pop_byte
                        wrbyte  data_8, dp_reg
                        call    #pop_word
                        wrword  data_16, x_reg
                        call    #pop_word
                        wrword  data_16, y_reg
                        call    #pop_word
                        wrword  data_16, u_reg
:pc_only                call    #pop_word
                        mov     pc, data_16
                        wrword  address, s_reg
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
rti_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'RESET overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
reset_ovl               mov     address, :reset_vector
                        call    #read_memory_word
                        mov     pc, data_16
                        jmp     #fetch
:reset_vector           long    $FFFE
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
reset_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------
'ABX overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
abx_ovl                 rdbyte  data_8, b_reg
                        rdword  data_16, x_reg
                        add     data_16, data_8
                        wrword  data_16, x_reg
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
abx_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'RTS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START

rts_ovl                 rdword  address, s_reg             'Get current stack pointer
                        call    #pop_word
                        mov     pc, data_16
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
rts_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'SWI overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
swi_ovl                 or      cc, #entire_flag
                        mov     data_8, pc
                        call    #push_word                  'FIXME Doing pushes in loops
                        mov     data_8, pc                  'would save space and overlay load time!
                        call    #push_word
                        rdword  data_8, u_reg
                        call    #push_word
                        rdword  data_8, y_reg
                        call    #push_word
                        rdword  data_8, x_reg
                        call    #push_word
                        rdbyte  data_8, dp_reg
                        call    #push_byte
                        rdbyte  data_8, b_reg
                        call    #push_byte
                        rdbyte  data_8, a_reg
                        call    #push_byte
                        mov     data_8, cc
                        call    #push_byte
                        wrword  address, s_reg
                        or      cc, #(fast_int_mask | interrupt_mask)
                        mov     address, :swi_vector
                        call    #read_memory_word
                        mov     pc, data_16
:swi_vector             long    $FFFA
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
swi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'SWI2 overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
swi2_ovl                or      cc, #entire_flag
                        mov     data_8, pc
                        call    #push_word                  'FIXME Doing pushes in loops
                        mov     data_8, pc                  'would save space and overlay load time!
                        call    #push_word
                        rdword  data_8, u_reg
                        call    #push_word
                        rdword  data_8, y_reg
                        call    #push_word
                        rdword  data_8, x_reg
                        call    #push_word
                        rdbyte  data_8, dp_reg
                        call    #push_byte
                        rdbyte  data_8, b_reg
                        call    #push_byte
                        rdbyte  data_8, a_reg
                        call    #push_byte
                        mov     data_8, cc
                        call    #push_byte
                        wrword  address, s_reg
                        mov     address, :swi2_vector
                        call    #read_memory_word
                        mov     pc, data_16
:swi2_vector            long    $FFF4
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
swi2_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'SWI3 overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
swi3_ovl                or      cc, #entire_flag
                        mov     data_8, pc
                        call    #push_word                  'FIXME Doing pushes in loops
                        mov     data_8, pc                  'would save space and overlay load time!
                        call    #push_word
                        rdword  data_8, u_reg
                        call    #push_word
                        rdword  data_8, y_reg
                        call    #push_word
                        rdword  data_8, x_reg
                        call    #push_word
                        rdbyte  data_8, dp_reg
                        call    #push_byte
                        rdbyte  data_8, b_reg
                        call    #push_byte
                        rdbyte  data_8, a_reg
                        call    #push_byte
                        mov     data_8, cc
                        call    #push_byte
                        wrword  address, s_reg
                        mov     address, :swi3_vector
                        call    #read_memory_word
                        mov     pc, data_16
:swi3_vector            long    $FFF2
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
swi3_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BRN overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
brn_ovl                 mov     alu, #0
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
brn_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BHI overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bhi_ovl                 test    cc, #carry_flag | zero_flag wz
                        mov     alu, #0        'FIXME Why is this not a mux ?
              if_z      mov     alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bhi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BLS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bls_ovl                 test    cc, #carry_flag | zero_flag wz
                        mov     alu, #0         'FIXME Why is this not a mux ?
              if_nz     mov     alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bls_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BHS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bhs_ovl                 test    cc, #carry_flag wz
                        muxz    alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bhs_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BLO overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
blo_ovl                 test    cc, #carry_flag wz
                        muxnz   alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
blo_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BNE overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bne_ovl                 test    cc, #zero_flag wz
                        muxz    alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bne_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BEQ overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
beq_ovl                 test    cc, #zero_flag wz
                        muxnz   alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
beq_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BVC overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bvc_ovl                 test    cc, #overflow_flag wz
                        muxz    alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bvc_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BVS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bvs_ovl                 test    cc, #overflow_flag wz
                        muxnz   alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bvs_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BPL overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bpl_ovl                 test    cc, #negative_flag wc
                        muxnc   alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bpl_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BMI overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bmi_ovl                 test    cc, #negative_flag wc
                        muxc    alu, #1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bmi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BGE overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bge_ovl                 test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        muxnc   alu, 1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bge_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BLT overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
blt_ovl                 test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        muxc    alu, 1
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
blt_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BGT overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
bgt_ovl                 test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        test    cc, #zero_flag wz                     'Prop zero  := Z
                        mov     alu, #0
              if_nz_or_nc mov    alu, #1                         'alu := Z AND (N XOR V)
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
bgt_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------
'BLE overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
ble_ovl                 test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        test    cc, #zero_flag wz                     'Prop zero  := Z
                        mov     alu, #0
              if_z_or_c mov     alu, #1                         'alu := Z OR (N XOR V)
                        jmp     #branch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
ble_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBRN overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbrn_ovl                mov     alu, #0
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbrn_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBHI overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbhi_ovl                test    cc, #carry_flag | zero_flag wz
                        mov     alu, #0        'FIXME Why is this not a mux ?
              if_z      mov     alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbhi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBLS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbls_ovl                test    cc, #carry_flag | zero_flag wz
                        mov     alu, #0         'FIXME Why is this not a mux ?
              if_nz     mov     alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbls_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBHS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbhs_ovl                test    cc, #carry_flag wz
                        muxz    alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbhs_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBLO overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lblo_ovl                test    cc, #carry_flag wz
                        muxnz   alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lblo_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBNE overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbne_ovl                test    cc, #zero_flag wz
                        muxz    alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbne_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBEQ overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbeq_ovl                test    cc, #zero_flag wz
                        muxnz   alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbeq_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBVC overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbvc_ovl                test    cc, #overflow_flag wz
                        muxz    alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbvc_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBVS overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbvs_ovl                test    cc, #overflow_flag wz
                        muxnz   alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbvs_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBPL overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbpl_ovl                test    cc, #negative_flag wc
                        muxnc   alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbpl_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBMI overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbmi_ovl                test    cc, #negative_flag wc
                        muxc    alu, #1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbmi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBGE overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbge_ovl                 test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        muxnc   alu, 1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbge_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBLT overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lblt_ovl                test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        muxc    alu, 1
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lblt_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBGT overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lbgt_ovl                test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        test    cc, #zero_flag wz                     'Prop zero  := Z
                        mov     alu, #0
              if_nz_or_nc mov    alu, #1                         'alu := Z AND (N XOR V)
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lbgt_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LBLE overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
lble_ovl                test    cc, #negative_flag | overflow_flag wc 'Prop carry := N XOR V
                        test    cc, #zero_flag wz                     'Prop zero  := Z
                        mov     alu, #0
              if_z_or_c mov     alu, #1                         'alu := Z OR (N XOR V)
                        jmp     #lbranch_conditional
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
lble_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ORCC overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
orcc_ovl                call    #get_byte_immediate_f
                        or      cc, data_8
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
orcc_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ANDCC overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
andcc_ovl               call    #get_byte_immediate_f
                        and     cc, data_8
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
andcc_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'SEX overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
sex_ovl                 rdword  data_16, d_reg             'Read A:B regs as D
                        shl     data_16, #24               'Sign extend B into A.
                        sar     data_16, #24 wz
                        jmp     #set_n_z_v0_double
                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
sex_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------



'---------------------------------------------------------------------------------------------------------
'out overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
out_ovl                 call    #get_word_immediate_f
#ifdef BANKED_MEMORY
                        cmp     address, #bank_sel_port wz 'Is this a write to the bank select port ?
              if_nz     jmp     #:not_bank_sel

                        rdbyte  data_8, a_reg              'Get the output data from A reg
                        movs    :set_offset, #:bank_table
                        add     :set_offset, data_8        'Index into bank offset table
:set_offset             mov     bank_offset, 0-0           'Set the bank_offset from a table entry
                        jmp     #fetch

:bank_table
:bank_0_offset          long    $000000                    'Bank 0 is the 48K below common
:bank_1_offset          long    $010000                    'Bank 1 is the 48K above common, i.e. above 64K
:bank_2_offset          long    $01C000                    'Following banks are 48K blocks following on from Bank 1
:bank_3_offset          long    $028000
:bank_4_offset          long    $034000

:not_bank_sel
#endif
                        wrword  data_16, io_port
                        rdbyte  data_8, a_reg
                        wrbyte  data_8, io_data            'Set the output data
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT
#ifdef TRIBLADEPROP
                        mov     dira, ram_dir_input        'RR20090321 (pass I/O to another cog)
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz      'Wait for the output done
              if_nz     jmp     #:wait
#ifdef TRIBLADEPROP
                        mov     dira, ram_dir_read         'RR20090321 (take I/O from another cog)
#endif
                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
out_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'in overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
in_ovl                  call    #get_byte_immediate_f
                        wrbyte  data_8, io_port
                        mov     data_8, #io_cmd_in         'Set I/O command to IN
#ifdef TRIBLADEPROP
                        mov     dira, ram_dir_input        'RR20090321 (pass I/O to another cog)
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz      'Wait for the input data
              if_nz     jmp     #:wait
#ifdef TRIBLADEPROP
                        mov     dira, ram_dir_read         'RR20090321 (take I/O from another cog)
#endif
                        rdbyte  data_8, io_data            'Read the input data to accumulator
                        wrbyte  data_8, a_reg
                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
in_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'break overlay. This is a "called" overlay.
                        org     OVERLAY_START
break_ovl               wrword  pc, pc_reg
                        wrbyte  cc, cc_reg
                        mov     data_8, #io_cmd_break      'Set I/O command to BREAK
#ifdef TRIBLADEPROP
                        mov     dira, ram_dir_input        'RR20090321 (pass I/O to another cog)
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz
              if_nz     jmp     #:wait
#ifdef TRIBLADEPROP
                        mov     dira, ram_dir_read         'RR20090321 (take I/O from another cog)
#endif
                        jmp     OVERLAY_LOAD_RET           'return(indirect) to calling routine

                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
break_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------





'---------------------------------------------------------------------------------------------------------
CON
'Overlay numbers.
'Note: These overlay numbers must be the in the order of the overlay param
'      entries in the overlay_table.
pshs_ovl_no  = $00
pshu_ovl_no  = $01
puls_ovl_no  = $02
pulu_ovl_no  = $03
mul_ovl_no   = $04
tfr_ovl_no   = $05
exg_ovl_no   = $06
daa_ovl_no   = $07
out_ovl_no   = $08
in_ovl_no    = $09
rti_ovl_no   = $0A
reset_ovl_no = $0B
abx_ovl_no   = $0C
rts_ovl_no   = $0D
swi_ovl_no   = $0E
swi2_ovl_no  = $0F
swi3_ovl_no  = $10
brn_ovl_no   = $11
bhi_ovl_no   = $12
bls_ovl_no   = $13
bhs_ovl_no   = $14
blo_ovl_no   = $15
bne_ovl_no   = $16
beq_ovl_no   = $17
bvc_ovl_no   = $18
bvs_ovl_no   = $19
bpl_ovl_no   = $1A
bmi_ovl_no   = $1B
bge_ovl_no   = $1C
blt_ovl_no   = $1D
bgt_ovl_no   = $1E
ble_ovl_no   = $1F
lbrn_ovl_no  = $20
lbhi_ovl_no  = $21
lbls_ovl_no  = $22
lbhs_ovl_no  = $23
lblo_ovl_no  = $24
lbne_ovl_no  = $25
lbeq_ovl_no  = $26
lbvc_ovl_no  = $27
lbvs_ovl_no  = $28
lbpl_ovl_no  = $29
lbmi_ovl_no  = $2A
lbge_ovl_no  = $2B
lblt_ovl_no  = $2C
lbgt_ovl_no  = $2D
lble_ovl_no  = $2E
orcc_ovl_no  = $2F
andcc_ovl_no = $30
sex_ovl_no   = $31

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
DAT
' Table of overlay parameters.
' Note: The overlay numbers defined in CON must be in the order
'       of the param entries here.
overlay_table
{00}pshs_ovl_           long    0-0             '\\ parameters for overlays (preset by spin code)
{01}pshu_ovl_           long    0-0             '||   bits 31..16 = hub END+3 address (must be even length)
{02}puls_ovl_           long    0-0             '||   bits  8..0  = cog END address
{03}pulu_ovl_           long    0-0             '||
{04}mul_ovl_            long    0-0             '||
{05}tfr_ovl_            long    0-0             '||
{06}exg_ovl_            long    0-0             '||
{07}daa_ovl_            long    0-0             '||
{08}out_ovl_            long    0-0             '||
{09}in_ovl_             long    0-0             '||
{0A}rti_ovl_            long    0-0             '||
{0B}reset_ovl_          long    0-0             '||
{0C}abx_ovl_            long    0-0             '||
{0D}rts_ovl_            long    0-0             '||
{0E}swi_ovl_            long    0-0             '||
{0F}swi2_ovl_           long    0-0             '||
{10}swi3_ovl_           long    0-0             '||
{11}brn_ovl_            long    0-0             '||
{12}bhi_ovl_            long    0-0             '||
{13}bls_ovl_            long    0-0             '||
{14}bhs_ovl_            long    0-0             '||
{15}blo_ovl_            long    0-0             '||
{16}bne_ovl_            long    0-0             '||
{17}beq_ovl_            long    0-0             '||
{18}bvc_ovl_            long    0-0             '||
{19}bvs_ovl_            long    0-0             '||
{1A}bpl_ovl_            long    0-0             '||
{1B}bmi_ovl_            long    0-0             '||
{1C}bge_ovl_            long    0-0             '||
{1D}blt_ovl_            long    0-0             '||
{1E}bgt_ovl_            long    0-0             '||
{1F}ble_ovl_            long    0-0             '||
{20}lbrn_ovl_           long    0-0             '||
{21}lbhi_ovl_           long    0-0             '||
{22}lbls_ovl_           long    0-0             '||
{23}lbhs_ovl_           long    0-0             '||
{24}lblo_ovl_           long    0-0             '||
{25}lbne_ovl_           long    0-0             '||
{26}lbeq_ovl_           long    0-0             '||
{27}lbvc_ovl_           long    0-0             '||
{28}lbvs_ovl_           long    0-0             '||
{29}lbpl_ovl_           long    0-0             '||
{2A}lbmi_ovl_           long    0-0             '||
{2B}lbge_ovl_           long    0-0             '||
{2C}lblt_ovl_           long    0-0             '||
{2D}lbgt_ovl_           long    0-0             '||
{2E}lble_ovl_           long    0-0             '||
{2F}orcc_ovl_           long    0-0             '||
{30}andcc_ovl_          long    0-0             '||
{31}sex_ovl_            long    0-0             '||
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------
dispatch_table_0                                                           'Op code,  Add. Mode.  Cl, S, HNZVC
{00}	LONG	get_byte_direct    + (neg_byte << 9) + (put_byte_ea << 18) 'NEG       Direct,      6, 2, uaaaa
{01}	LONG	vector_overlay     + (out_ovl_no <<9)	                   'Illegal,               1, 1, uuuuu
{02}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{03}	LONG	get_byte_direct    + (com_byte << 9) + (put_byte_ea << 18) 'COM       Direct,      6, 2, -aa01
{04}	LONG	get_byte_direct    + (lsr_byte << 9) + (put_byte_ea << 18) 'LSR       Direct,      6, 2, -0a-s
{05}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{06}	LONG	get_byte_direct    + (ror_byte << 9) + (put_byte_ea << 18) 'ROR       Direct,      6, 2, -aa-s
{07}	LONG	get_byte_direct    + (asr_byte << 9) + (put_byte_ea << 18) 'ASR       Direct,      6, 2, uaa-s
{08}	LONG	get_byte_direct    + (lsl_byte << 9) + (put_byte_ea << 18) 'LSL/ASL   Direct,      6, 2, naaas
{09}	LONG	get_byte_direct    + (rol_byte << 9) + (put_byte_ea << 18) 'ROL       Direct,      6, 2, -aaas
{0A}	LONG	get_byte_direct    + (dec_byte << 9) + (put_byte_ea << 18) 'DEC       Direct,      6, 2, -aaa-
{0B}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{0C}	LONG	get_byte_direct    + (inc_byte << 9) + (put_byte_ea << 18) 'INC       Direct,      6, 2, -aaa-
{0D}	LONG	get_byte_direct    + (tst_byte << 9)                       'TST       Direct,      6, 2, -aa0-
{0E}	LONG	get_ea_direct      + (jmp_ << 9)                           'JMP       Direct,      3, 2, -----
{0F}	LONG	get_byte_direct    + (clr_byte << 9) + (put_byte_ea << 18) 'CLR       Direct,      6, 2, -0100

{10}	LONG	page1	                                                   'PAGE1+    Variant,     1, 1, +++++
{11}	LONG	page2	                                                   'PAGE2+    Variant,     1, 1, +++++
{12}	LONG	fetch                                                      'NOP       Inherent,    2, 1, -----
{13}	LONG	not_implemented	                                           'SYNC      Inherent,    2, 1, -----
{14}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{15}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{16}	LONG	lbranch	                                                   'LBRA      Relative,    5, 3, -----
{17}	LONG	lbsr                                                       'LBSR      Relative,    9, 3, -----
{18}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{19}	LONG	vector_overlay     + (daa_ovl_no  << 9)                    'DAA       Inherent,    2, 1, -aa0a
{1A}	LONG	vector_overlay     + (orcc_ovl_no << 9)                    'ORCC      Immediate,   3, 2, ddddd
{1B}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{1C}	LONG	vector_overlay     + (andcc_ovl_no << 9)                   'ANDCC     Immediate,   3, 2, ddddd
{1D}	LONG	vector_overlay     + (sex_ovl_no <<9)                      'SEX       Inherent,    2, 1, -aa0-
{1E}	LONG	vector_overlay     + (exg_ovl_no << 9)                     'EXG       Immediate,   8, 2, ccccc
{1F}	LONG	vector_overlay     + (tfr_ovl_no << 9)                     'TFR       Inherent,    7, 2, ccccc

{20}	LONG	branch                                                     'BRA       Relative,    3, 2, -----
{21}	LONG	vector_overlay     + (brn_ovl_no << 9)                     'BRN       Relative,    3, 2, -----
{22}	LONG	vector_overlay     + (bhi_ovl_no << 9)                     'BHI       Relative,    3, 2, -----
{23}	LONG	vector_overlay     + (bls_ovl_no << 9)                     'BLS       Relative,    3, 2, -----
{24}	LONG	vector_overlay     + (bhs_ovl_no << 9)                     'BHS/BCC   Relative,    3, 2, -----
{25}	LONG	vector_overlay     + (blo_ovl_no << 9)                     'BLO/BCS   Relative,    3, 2, -----
{26}	LONG	vector_overlay     + (bne_ovl_no << 9)                     'BNE       Relative,    3, 2, -----
{27}	LONG	vector_overlay     + (beq_ovl_no << 9)                     'BEQ       Relative,    3, 2, -----
{28}	LONG	vector_overlay     + (bvc_ovl_no << 9)                     'BVC       Relative,    3, 2, -----
{29}	LONG	vector_overlay     + (bvs_ovl_no << 9)                     'BVS       Relative,    3, 2, -----
{2A}	LONG	vector_overlay     + (bpl_ovl_no << 9)                     'BPL       Relative,    3, 2, -----
{2B}	LONG	vector_overlay     + (bmi_ovl_no << 9)                     'BMI       Relative,    3, 2, -----
{2C}	LONG	vector_overlay     + (bge_ovl_no << 9)                     'BGE       Relative,    3, 2, -----
{2D}	LONG	vector_overlay     + (blt_ovl_no << 9)                     'BLT       Relative,    3, 2, -----
{2E}	LONG	vector_overlay     + (bgt_ovl_no << 9)                     'BGT       Relative,    3, 2, -----
{2F}	LONG	vector_overlay     + (ble_ovl_no << 9)                     'BLE       Relative,    3, 2, -----

{30}	LONG	get_ea_indexed     + (select_reg_x << 9) + (lea << 18)     'LEAX      Indexed,     4, 2, --a--
{31}	LONG	get_ea_indexed     + (select_reg_y << 9) + (lea << 18)	   'LEAY      Indexed,     4, 2, --a--
{32}	LONG	get_ea_indexed     + (select_reg_s << 9) + (lea << 18)	   'LEAS      Indexed,     4, 2, -----
{33}	LONG	get_ea_indexed     + (select_reg_u << 9) + (lea << 18)     'LEAU      Indexed,     4, 2, -----
{34}	LONG	vector_overlay     + (pshs_ovl_no  << 9)                   'PSHS      Inherent,    5, 2, -----
{35}	LONG	vector_overlay     + (puls_ovl_no  << 9)                   'PULS      Inherent,    5, 2, ccccc
{36}	LONG	vector_overlay     + (pshu_ovl_no  << 9)                   'PSHU      Inherent,    5, 2, -----
{37}	LONG	vector_overlay     + (pulu_ovl_no  << 9)	           'PULU      Inherent,    5, 2, ccccc
{38}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{39}	LONG	vector_overlay     + (rts_ovl_no << 9)                     'RTS       Inherent,    5, 1, -----
{3A}	LONG	vector_overlay     + (abx_ovl_no << 9)                     'ABX       Inherent,    3, 1, -----
{3B}	LONG	vector_overlay     + (rti_ovl_no << 9)                     'RTI       Inherent, 6/15, 1, -----
{3C}	LONG	not_implemented                                            'CWAI      Inherent,   21, 2, ddddd
{3D}	LONG	vector_overlay     + (mul_ovl_no << 9)                     'MUL       Inherent,   11, 1, --a-a
{3E}	LONG	vector_overlay     + (reset_ovl_no << 9)                   'RESET     Inherent,    *, 1, *****
{3F}	LONG	vector_overlay     + (swi_ovl_no << 9)                     'SWI       Inherent,   19, 1, -----

{40}	LONG	get_byte_a         + (neg_byte << 9) + (put_byte_a << 18)  'NEGA      Inherent,    2, 1, uaaaa
{41}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{42}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{43}	LONG	get_byte_a         + (com_byte << 9) + (put_byte_a << 18)  'COMA      Inherent,    2, 1, -aa01
{44}	LONG	get_byte_a         + (lsr_byte << 9) + (put_byte_a << 18)  'LSRA      Inherent,    2, 1, -0a-s
{45}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{46}	LONG	get_byte_a         + (ror_byte << 9) + (put_byte_a << 18)  'RORA      Inherent,    2, 1, -aa-s
{47}	LONG	get_byte_a         + (asr_byte << 9) + (put_byte_a << 18)  'ASRA      Inherent,    2, 1, uaa-s
{48}	LONG	get_byte_a         + (lsl_byte << 9) + (put_byte_a << 18)  'LSLA/ASLA Inherent,    2, 1, naaas
{49}	LONG	get_byte_a         + (rol_byte << 9) + (put_byte_a << 18)  'ROLA      Inherent,    2, 1, -aaas
{4A}	LONG	get_byte_a         + (dec_byte << 9) + (put_byte_a << 18)  'DECA      Inherent,    2, 1, -aaa-
{4B}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{4C}	LONG	get_byte_a         + (inc_byte << 9) + (put_byte_a << 18)  'INCA      Inherent,    2, 1, -aaa-
{4D}	LONG	get_byte_a         + (tst_byte << 9)                       'TSTA      Inherent,    2, 1, -aa0-
{4E}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{4F}	LONG	get_byte_a         + (clr_byte << 9) + (put_byte_a << 18)  'CLRA      Inherent,    2, 1, -0100

{50}	LONG    get_byte_b         + (neg_byte << 9) + (put_byte_b << 18)  'NEGB      Inherent,    2, 1, uaaaa
{51}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{52}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{53}	LONG	get_byte_b         + (com_byte << 9) + (put_byte_b << 18)  'COMB      Inherent,    2, 1, -aa01
{54}	LONG	get_byte_b         + (lsr_byte << 9) + (put_byte_b << 18)  'LSRB      Inherent,    2, 1, -0a-s
{55}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{56}	LONG	get_byte_b         + (ror_byte << 9) + (put_byte_b << 18)  'RORB      Inherent,    2, 1, -aa-s
{57}	LONG	get_byte_b         + (asr_byte << 9) + (put_byte_b << 18)  'ASRB      Inherent,    2, 1, uaa-s
{58}	LONG	get_byte_b         + (lsl_byte << 9) + (put_byte_b << 18)  'LSLB/ASLB Inherent,    2, 1, naaas
{59}	LONG	get_byte_b         + (rol_byte << 9) + (put_byte_b << 18)  'ROLB      Inherent,    2, 1, -aaas
{5A}	LONG	get_byte_b         + (dec_byte << 9) + (put_byte_b << 18)  'DECB      Inherent,    2, 1, -aaa-
{5B}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{5C}	LONG	get_byte_b         + (inc_byte << 9) + (put_byte_b << 18)  'INCB      Inherent,    2, 1, -aaa-
{5D}	LONG    get_byte_b         + (tst_byte << 9)                       'TSTB      Inherent,    2, 1, -aa0-
{5E}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{5F}	LONG	get_byte_b         + (clr_byte << 9) + (put_byte_b << 18)  'CLRB      Inherent,    2, 1, -0100

{60}	LONG	get_byte_indexed   + (neg_byte << 9) + (put_byte_ea << 18) 'NEG       Indexed,     6, 2, uaaaa
{61}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{62}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{63}	LONG	get_byte_indexed   + (com_byte << 9) + (put_byte_ea << 18) 'COM       Indexed,     6, 2, -aa01
{64}	LONG	get_byte_indexed   + (lsr_byte << 9) + (put_byte_ea << 18) 'LSR       Indexed,     6, 2, -0a-s
{65}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{66}	LONG	get_byte_indexed   + (ror_byte << 9) + (put_byte_ea << 18) 'ROR       Indexed,     6, 2, -aa-s
{67}	LONG	get_byte_indexed   + (asr_byte << 9) + (put_byte_ea << 18) 'ASR       Indexed,     6, 2, uaa-s
{68}	LONG	get_byte_indexed   + (lsl_byte << 9) + (put_byte_ea << 18) 'LSL/ASL   Indexed,     6, 2, naaas
{69}	LONG	get_byte_indexed   + (rol_byte << 9) + (put_byte_ea << 18) 'ROL       Indexed,     6, 2, -aaas
{6A}	LONG	get_byte_indexed   + (dec_byte << 9) + (put_byte_ea << 18) 'DEC       Indexed,     6, 2, -aaa-
{6B}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{6C}	LONG	get_byte_indexed   + (inc_byte << 9) + (put_byte_ea << 18) 'INC       Indexed,     6, 2, -aaa-
{6D}	LONG	get_byte_indexed   + (tst_byte << 9)                       'TST       Indexed,     6, 2, -aa0-
{6E}	LONG	get_ea_indexed     + (jmp_ << 9)                           'JMP       Indexed,     3, 2, -----
{6F}	LONG	get_byte_indexed   + (clr_byte << 9) + (put_byte_ea << 18) 'CLR       Indexed,     6, 2, -0100

{70}	LONG	get_byte_extended  + (neg_byte << 9) + (put_byte_ea << 18) 'NEG       Extended,    7, 3, uaaaa
{71}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{72}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{73}	LONG	get_byte_extended  + (com_byte << 9) + (put_byte_ea << 18) 'COM       Extended,    7, 3, -aa01
{74}	LONG	get_byte_extended  + (lsr_byte << 9) + (put_byte_ea << 18) 'LSR       Extended,    7, 3, -0a-s
{75}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{76}	LONG	get_byte_extended  + (ror_byte << 9) + (put_byte_ea << 18) 'ROR       Extended,    7, 3, -aa-s
{77}	LONG	get_byte_extended  + (asr_byte << 9) + (put_byte_ea << 18) 'ASR       Extended,    7, 3, uaa-s
{78}	LONG	get_byte_extended  + (lsl_byte << 9) + (put_byte_ea << 18) 'LSL/ASL   Extended,    7, 3, naaas
{79}	LONG	get_byte_extended  + (rol_byte << 9) + (put_byte_ea << 18) 'ROL       Extended,    7, 3, -aaas
{7A}	LONG	get_byte_extended  + (dec_byte << 9) + (put_byte_ea << 18) 'DEC       Extended,    7, 3, -aaa-
{7B}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{7C}	LONG	get_byte_extended  + (inc_byte << 9) + (put_byte_ea << 18) 'INC       Extended,    7, 3, -aaa-
{7D}	LONG	get_byte_extended  + (tst_byte << 9)                       'TST       Extended,    7, 3, -aa0-
{7E}	LONG	get_ea_extended    + (jmp_ << 9)                           'JMP       Extended,    3, 3, -----
{7F}	LONG	get_byte_extended  + (clr_byte << 9) + (put_byte_ea << 18) 'CLR       Extended,    7, 3, -0100

{80}	LONG	get_byte_immediate + (select_reg_a << 9) + (sub_ << 18)    'SUBA      Immediate,   2, 2, uaaaa
{81}	LONG	get_byte_immediate + (select_reg_a << 9) + (cmp_ << 18)    'CMPA      Immediate,   2, 2, uaaaa
{82}	LONG	get_byte_immediate + (select_reg_a << 9) + (sbc << 18)     'SBCA      Immediate,   2, 2, uaaaa
{83}	LONG	get_word_immediate + (select_reg_d << 9) + (subd << 18)    'SUBD      Immediate,   4, 3, -aaaa
{84}	LONG	get_byte_immediate + (select_reg_a << 9) + (and_ << 18)    'ANDA      Immediate,   2, 2, -aa0-
{85}	LONG	get_byte_immediate + (select_reg_a << 9) + (bit << 18)     'BITA      Immediate,   2, 2, -aa0-
{86}	LONG	get_byte_immediate + (select_reg_a << 9) + (ld << 18)	   'LDA       Immediate,   2, 2, -aa0-
{87}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{88}	LONG	get_byte_immediate + (select_reg_a << 9) + (eor << 18)     'EORA      Immediate,   2, 2, -aa0-
{89}	LONG	get_byte_immediate + (select_reg_a << 9) + (adc << 18)     'ADCA      Immediate,   2, 2, aaaaa
{8A}	LONG	get_byte_immediate + (select_reg_a << 9) + (or_ << 18)     'ORA       Immediate,   2, 2, -aa0-
{8B}	LONG	get_byte_immediate + (select_reg_a << 9) + (add_ << 18)    'ADDA      Immediate,   2, 2, aaaaa
{8C}	LONG	get_word_immediate + (select_reg_x << 9) + (cmpd << 18)    'CMPX      Immediate,   4, 3, -aaaa
{8D}	LONG	bsr                                                        'BSR       Relative,    7, 2, -----
{8E}	LONG	get_word_immediate + (select_reg_x << 9) + (ldd << 18)     'LDX       Immediate,   3, 3, -aa0-
{8F}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu

{90}	LONG	get_byte_direct    + (select_reg_a << 9) + (sub_ << 18)    'SUBA      Direct,      4, 2, uaaaa
{91}	LONG	get_byte_direct    + (select_reg_a << 9) + (cmp_ << 18)    'CMPA      Direct,      4, 2, uaaaa
{92}	LONG	get_byte_direct    + (select_reg_a << 9) + (sbc << 18)     'SBCA      Direct,      4, 2, uaaaa
{93}	LONG	get_word_direct    + (select_reg_d << 9) + (subd << 18)    'SUBD      Direct,      6, 2, -aaaa
{94}	LONG	get_byte_direct    + (select_reg_a << 9) + (and_ << 18)    'ANDA      Direct,      4, 2, -aa0-
{95}	LONG	get_byte_direct    + (select_reg_a << 9) + (bit << 18)     'BITA      Direct,      4, 2, -aa0-
{96}	LONG	get_byte_direct    + (select_reg_a << 9) + (ld << 18)      'LDA       Direct,      4, 2, -aa0-
{97}	LONG	get_ea_direct	   + (select_reg_a << 9) + (st << 18)      'STA       Direct,      4, 2, -aa0-
{98}	LONG	get_byte_direct    + (select_reg_a << 9) + (eor << 18)     'EORA      Direct,      4, 2, -aa0-
{99}	LONG	get_byte_direct    + (select_reg_a << 9) + (adc << 18)     'ADCA      Direct,      4, 2, aaaaa
{9A}	LONG	get_byte_direct    + (select_reg_a << 9) + (or_ << 18)     'ORA       Direct,      4, 2, -aa0-
{9B}	LONG	get_byte_direct    + (select_reg_a << 9) + (add_ << 18)    'ADDA      Direct,      4, 2, aaaaa
{9C}	LONG	get_word_direct    + (select_reg_x << 9) + (cmpd << 18)    'CMPX      Direct,      6, 2, -aaaa
{9D}	LONG	get_ea_direct	   + (jsr << 9)                            'JSR       Direct,      7, 2, -----
{9E}	LONG	get_word_direct    + (select_reg_x << 9) + (ldd << 18)     'LDX       Direct,      5, 2, -aa0-
{9F}	LONG	get_ea_direct	   + (select_reg_x << 9) + (std << 18)     'STX       Direct,      5, 2, -aa0-

{A0}	LONG	get_byte_indexed   + (select_reg_a << 9) + (sub_ << 18)    'SUBA      Indexed,     4, 2, uaaaa
{A1}	LONG	get_byte_indexed   + (select_reg_a << 9) + (cmp_ << 18)    'CMPA      Indexed,     4, 2, uaaaa
{A2}	LONG	get_byte_indexed   + (select_reg_a << 9) + (sbc << 18)     'SBCA      Indexed,     4, 2, uaaaa
{A3}	LONG	get_word_indexed   + (select_reg_d << 9) + (subd << 18)    'SUBD      Indexed,     6, 2, -aaaa
{A4}	LONG	get_byte_indexed   + (select_reg_a << 9) + (and_ << 18)    'ANDA      Indexed,     4, 2, -aa0-
{A5}	LONG	get_byte_indexed   + (select_reg_a << 9) + (bit << 18)     'BITA      Indexed,     4, 2, -aa0-
{A6}	LONG	get_byte_indexed   + (select_reg_a << 9) + (ld << 18)      'LDA       Indexed,     4, 2, -aa0-
{A7}	LONG	get_ea_indexed	   + (select_reg_a << 9) + (st << 18)      'STA       Indexed,     4, 2, -aa0-
{A8}	LONG	get_byte_indexed   + (select_reg_a << 9) + (eor << 18)     'EORA      Indexed,     4, 2, -aa0-
{A9}	LONG	get_byte_indexed   + (select_reg_a << 9) + (adc << 18)     'ADCA      Indexed,     4, 2, aaaaa
{AA}	LONG	get_byte_indexed   + (select_reg_a << 9) + (or_ << 18)     'ORA       Indexed,     4, 2, -aa0-
{AB}	LONG	get_byte_indexed   + (select_reg_a << 9) + (add_ << 18)    'ADDA      Indexed,     4, 2, aaaaa
{AC}	LONG	get_word_indexed   + (select_reg_x << 9) + (cmpd << 18)    'CMPX      Indexed,     6, 2, -aaaa
{AD}	LONG	get_ea_indexed     + (jsr << 9)                            'JSR       Indexed,     7, 2, -----
{AE}	LONG	get_word_indexed   + (select_reg_x << 9) + (ldd << 18)     'LDX       Indexed,     5, 2, -aa0-
{AF}	LONG    get_ea_indexed	   + (select_reg_x << 9) + (std << 18)     'STX       Indexed,     5, 2, -aa0-

{B0}	LONG	get_byte_extended  + (select_reg_a << 9) + (sub_ << 18)    'SUBA      Extended,    5, 3, uaaaa
{B1}	LONG	get_byte_extended  + (select_reg_a << 9) + (cmp_ << 18)    'CMPA      Extended,    5, 3, uaaaa
{B2}	LONG	get_byte_extended  + (select_reg_a << 9) + (sbc << 18)     'SBCA      Extended,    5, 3, uaaaa
{B3}	LONG	get_word_extended  + (select_reg_d << 9) + (subd << 18)    'SUBD      Extended,    7, 3, -aaaa
{B4}	LONG	get_byte_extended  + (select_reg_a << 9) + (and_ << 18)    'ANDA      Extended,    5, 3, -aa0-
{B5}	LONG	get_byte_extended  + (select_reg_a << 9) + (bit << 18)     'BITA      Extended,    5, 3, -aa0-
{B6}	LONG	get_byte_extended  + (select_reg_a << 9) + (ld << 18)      'LDA       Extended,    5, 3, -aa0-
{B7}	LONG	get_ea_extended	   + (select_reg_a << 9) + (st << 18)      'STA       Extended,    5, 3, -aa0-
{B8}	LONG	get_byte_extended  + (select_reg_a << 9) + (eor << 18)     'EORA      Extended,    5, 3, -aa0-
{B9}	LONG	get_byte_extended  + (select_reg_a << 9) + (adc << 18)     'ADCA      Extended,    5, 3, aaaaa
{BA}	LONG	get_byte_extended  + (select_reg_a << 9) + (or_ << 18)     'ORA       Extended,    5, 3, -aa0-
{BB}	LONG	get_byte_extended  + (select_reg_a << 9) + (add_ << 18)    'ADDA      Extended,    5, 3, aaaaa
{BC}	LONG	get_word_extended  + (select_reg_x << 9) + (cmpd << 18)    'CMPX      Extended,    7, 3, -aaaa
{BD}	LONG	get_ea_extended    + (jsr << 9)                            'JSR       Extended,    8, 3, -----
{BE}	LONG	get_word_extended  + (select_reg_x << 9) +  (ldd << 18)    'LDX       Extended,    6, 3, -aa0-
{BF}	LONG	get_ea_extended    + (select_reg_x << 9) +  (std << 18)    'STX       Extended,    6, 3, -aa0-

{C0}	LONG	get_byte_immediate + (select_reg_b << 9) + (sub_ << 18)    'SUBB      Immediate,   2, 2, uaaaa
{C1}	LONG	get_byte_immediate + (select_reg_b << 9) + (cmp_ << 18)	   'CMPB      Immediate,   2, 2, uaaaa
{C2}	LONG	get_byte_immediate + (select_reg_b << 9) + (sbc << 18)     'SBCB      Immediate,   2, 2, uaaaa
{C3}	LONG	get_word_immediate + (select_reg_d << 9) + (addd << 18)    'ADDD      Immediate,   4, 3, -aaaa
{C4}	LONG	get_byte_immediate + (select_reg_b << 9) + (and_ << 18)    'ANDB      Immediate,   2, 2, -aa0-
{C5}	LONG	get_byte_immediate + (select_reg_b << 9) + (bit << 18)     'BITB      Immediate,   2, 2, -aa0-
{C6}	LONG	get_byte_immediate + (select_reg_b << 9) + (ld << 18)      'LDB       Immediate,   2, 2, -aa0-
{C7}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{C8}	LONG	get_byte_immediate + (select_reg_b << 9) + (eor << 18)     'EORB      Immediate,   2, 2, -aa0-
{C9}	LONG	get_byte_immediate + (select_reg_b << 9) + (adc << 18)     'ADCB      Immediate,   2, 2, aaaaa
{CA}	LONG	get_byte_immediate + (select_reg_b << 9) + (or_ << 18)     'ORB       Immediate,   2, 2, -aa0-
{CB}	LONG	get_byte_immediate + (select_reg_b << 9) + (add_ << 18)    'ADDB      Immediate,   2, 2, aaaaa
{CC}	LONG	get_word_immediate + (select_reg_d << 9) + (ldd << 18)     'LDD       Immediate,   3, 3, -aa0-
{CD}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu
{CE}	LONG	get_word_immediate + (select_reg_u << 9) + (ldd << 18)     'LDU       Immediate,   3, 3, -aa0-
{CF}	LONG	illegal	                                                   'Illegal,               1, 1, uuuuu

{D0}	LONG	get_byte_direct    + (select_reg_b << 9) + (sub_ << 18)    'SUBB      Direct,      4, 2, uaaaa
{D1}	LONG	get_byte_direct    + (select_reg_b << 9) + (cmp_ << 18)    'CMPB      Direct,      4, 2, uaaaa
{D2}	LONG	get_byte_direct    + (select_reg_b << 9) + (sbc << 18)     'SBCB      Direct,      4, 2, uaaaa
{D3}	LONG	get_byte_direct    + (select_reg_d << 9) + (addd << 18)    'ADDD      Direct,      6, 2, -aaaa
{D4}	LONG	get_byte_direct    + (select_reg_b << 9) + (and_ << 18)    'ANDB      Direct,      4, 2, -aa0-
{D5}	LONG	get_byte_direct    + (select_reg_b << 9) + (bit << 18)     'BITB      Direct,      4, 2, -aa0-
{D6}	LONG	get_byte_direct    + (select_reg_b << 9) + (ld << 18)      'LDB       Direct,      4, 2, -aa0-
{D7}	LONG    get_ea_direct      + (select_reg_b << 9) + (st << 18)      'STB       Direct,      4, 2, -aa0-
{D8}	LONG	get_byte_direct    + (select_reg_b << 9) + (eor << 18)     'EORB      Direct,      4, 2, -aa0-
{D9}	LONG	get_byte_direct    + (select_reg_b << 9) + (adc << 18)     'ADCB      Direct,      4, 2, aaaaa
{DA}	LONG	get_byte_direct    + (select_reg_b << 9) + (or_ << 18)     'ORB       Direct,      4, 2, -aa0-
{DB}	LONG	get_byte_direct    + (select_reg_b << 9) + (add_ << 18)    'ADDB      Direct,      4, 2, aaaaa
{DC}	LONG	get_word_direct    + (select_reg_d << 9) + (ldd << 18)     'LDD       Direct,      5, 2, -aa0-
{DD}	LONG	get_ea_direct      + (select_reg_d << 9) + (std << 18)     'STD       Direct,      5, 2, -aa0-
{DE}	LONG	get_word_direct    + (select_reg_u << 9) + (ldd << 18)     'LDU       Direct,      5, 2, -aa0-
{DF}	LONG	get_ea_direct      + (select_reg_u << 9) + (std << 18)     'STU       Direct,      5, 2, -aa0-

{E0}	LONG	get_byte_indexed   + (select_reg_b << 9) + (sub_ << 18)    'SUBB      Indexed,     4, 2, uaaaa
{E1}	LONG	get_byte_indexed   + (select_reg_b << 9) + (cmp_ << 18)    'CMPB      Indexed,     4, 2, uaaaa
{E2}	LONG	get_byte_indexed   + (select_reg_b << 9) + (sbc  << 18)    'SBCB      Indexed,     4, 2, uaaaa
{E3}	LONG	get_word_indexed   + (select_reg_b << 9) + (addd << 18)    'ADDD      Indexed,     6, 2, -aaaa
{E4}	LONG	get_byte_indexed   + (select_reg_b << 9) + (and_ << 18)    'ANDB      Indexed,     4, 2, -aa0-
{E5}	LONG	get_byte_indexed   + (select_reg_b << 9) + (bit << 18)     'BITB      Indexed,     4, 2, -aa0-
{E6}	LONG	get_byte_indexed   + (select_reg_b << 9) + (ld << 18)      'LDB       Indexed,     4, 2, -aa0-
{E7}	LONG	get_ea_indexed	   + (select_reg_b << 9) + (st << 18)      'STB       Indexed,     4, 2, -aa0-
{E8}	LONG	get_byte_indexed   + (select_reg_b << 9) + (eor << 18)     'EORB      Indexed,     4, 2, -aa0-
{E9}	LONG	get_byte_indexed   + (select_reg_b << 9) + (adc << 18)     'ADCB      Indexed,     4, 2, aaaaa
{EA}	LONG	get_byte_indexed   + (select_reg_b << 9) + (or_ << 18)     'ORB       Indexed,     4, 2, -aa0-
{EB}	LONG	get_byte_indexed   + (select_reg_b << 9) + (add_ << 18)    'ADDB      Indexed,     4, 2, aaaaa
{EC}	LONG	get_word_indexed   + (select_reg_d << 9) + (ldd << 18)     'LDD       Indexed,     5, 2, -aa0-
{ED}	LONG	get_ea_indexed     + (select_reg_d << 9) + (std << 18)     'STD       Indexed,     5, 2, -aa0-
{EE}	LONG	get_word_indexed   + (select_reg_u << 9) + (ldd << 18)     'LDU       Indexed,     5, 2, -aa0-
{EF}	LONG	get_ea_indexed     + (select_reg_b << 9) + (std << 18)     'STU       Indexed,     5, 2, -aa0-

{F0}	LONG	get_byte_extended  + (select_reg_b << 9) + (sub_ << 18)    'SUBB      Extended,    5, 3, uaaaa
{F1}	LONG	get_byte_extended  + (select_reg_b << 9) + (cmp_ << 18)    'CMPB      Extended,    5, 3, uaaaa
{F2}	LONG	get_byte_extended  + (select_reg_b << 9) + (sbc << 18)     'SBCB      Extended,    5, 3, uaaaa
{F3}	LONG	get_word_extended  + (select_reg_d << 9) + (addd << 18)    'ADDD      Extended,    7, 3, -aaaa
{F4}	LONG	get_byte_extended  + (select_reg_b << 9) + (and_ << 18)    'ANDB      Extended,    5, 3, -aa0-
{F5}	LONG	get_byte_extended  + (select_reg_b << 9) + (bit << 18)     'BITB      Extended,    5, 3, -aa0-
{F6}	LONG	get_byte_extended  + (select_reg_b << 9) + (ld << 18)      'LDB       Extended,    5, 3, -aa0-
{F7}	LONG	get_ea_extended    + (select_reg_b << 9) + (st << 18)      'STB       Extended,    5, 3, -aa0-
{F8}	LONG	get_byte_extended  + (select_reg_b << 9) + (eor << 18)     'EORB      Extended,    5, 3, -aa0-
{F9}	LONG	get_byte_extended  + (select_reg_b << 9) + (adc << 18)     'ADCB      Extended,    5, 3, aaaaa
{FA}	LONG	get_byte_extended  + (select_reg_b << 9) + (or_ << 18)     'ORB       Extended,    5, 3, -aa0-
{FB}	LONG	get_byte_extended  + (select_reg_b << 9) + (add_ << 18)    'ADDB      Extended,    5, 3, aaaaa
{FC}	LONG	get_word_extended  + (select_reg_d << 9) + (ldd << 18)     'LDD       Extended,    6, 3, -aa0-
{FD}	LONG	get_ea_extended    + (select_reg_d << 9) + (std << 18)     'STD       Extended,    6, 3, -aa0-
{FE}	LONG	get_word_extended  + (select_reg_u << 9) + (ldd << 18)     'LDU       Extended,    6, 3, -aa0-
{FF}	LONG	get_ea_extended	   + (select_reg_u << 9) + (std << 18)     'STU       Extended,    6, 3, -aa0-
'---------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------
'Dispatch table for ops with a Page I prefix byte.
'N.B. This table also contains the handfull of Page II prefix ops. They have been added to free spaces
'of the Page 1 opcodes and can be dispatched to by subracting 1 from the Page II opcode.
'The consequence of this is that some illegal Page I ops will actually dispatch to some valid Page II op.
'Many illegal Page II ops will dispatch to valid Page I ops.
'Still if your 6809 code relies on illeagal ops you get what you deserve.
'This is all done in order to avoid having another, almost empty, dispatch table.
dispatch_table_1
{00}	LONG	illegal	                                                   'Illegal
{01}	LONG	illegal	                                                   'Illegal
{02}	LONG	illegal	                                                   'Illegal
{03}	LONG	illegal	                                                   'Illegal
{04}	LONG	illegal	                                                   'Illegal
{05}	LONG	illegal	                                                   'Illegal
{06}	LONG	illegal	                                                   'Illegal
{07}	LONG	illegal	                                                   'Illegal
{08}	LONG	illegal	                                                   'Illegal
{09}	LONG	illegal	                                                   'Illegal
{0A}	LONG	illegal	                                                   'Illegal
{0B}	LONG	illegal	                                                   'Illegal
{0C}	LONG	illegal	                                                   'Illegal
{0D}	LONG	illegal	                                                   'Illegal
{0E}	LONG	illegal	                                                   'Illegal
{0F}	LONG	illegal	                                                   'Illegal

{10}	LONG	illegal	                                                   'Illegal
{11}	LONG	illegal	                                                   'Illegal
{12}	LONG	illegal	                                                   'Illegal
{13}	LONG	illegal	                                                   'Illegal
{14}	LONG	illegal	                                                   'Illegal
{15}	LONG	illegal	                                                   'Illegal
{16}	LONG	illegal	                                                   'Illegal
{17}	LONG	illegal	                                                   'Illegal
{18}	LONG	illegal	                                                   'Illegal
{19}	LONG	illegal	                                                   'Illegal
{1A}	LONG	illegal	                                                   'Illegal
{1B}	LONG	illegal	                                                   'Illegal
{1C}	LONG	illegal	                                                   'Illegal
{1D}	LONG	illegal	                                                   'Illegal
{1E}	LONG	illegal	                                                   'Illegal
{1F}	LONG	illegal	                                                   'Illegal

{20}	LONG	illegal	                                                   'Illegal
{21}	LONG	vector_overlay     + (lbrn_ovl_no << 9)                    'LBRN      Relative, 5(6), 4, -----
{22}	LONG	vector_overlay     + (lbhi_ovl_no << 9)                    'LBHI      Relative, 5(6), 4, -----
{23}	LONG	vector_overlay     + (lbls_ovl_no << 9)                    'LBLS      Relative, 5(6), 4, -----
{24}	LONG	vector_overlay     + (lbhs_ovl_no << 9)                    'LBHS/LBCC Relative, 5(6), 4, -----
{25}	LONG	vector_overlay     + (lblo_ovl_no << 9)                    'LBLO/LBCS Relative, 5(6), 4, -----
{26}	LONG	vector_overlay     + (lbne_ovl_no << 9)                    'LBNE      Relative, 5(6), 4, -----
{27}	LONG	vector_overlay     + (lbeq_ovl_no << 9)                    'LBEQ      Relative, 5(6), 4, -----
{28}	LONG	vector_overlay     + (lbvc_ovl_no << 9)                    'LBVC      Relative, 5(6), 4, -----
{29}	LONG	vector_overlay     + (lbvs_ovl_no << 9)                    'LBVS      Relative, 5(6), 4, -----
{2A}	LONG	vector_overlay     + (lbpl_ovl_no << 9)                    'LBPL      Relative, 5(6), 4, -----
{2B}	LONG	vector_overlay     + (lbmi_ovl_no << 9)                    'LBMI      Relative, 5(6), 4, -----
{2C}	LONG	vector_overlay     + (lbge_ovl_no << 9)                    'LBGE      Relative, 5(6), 4, -----
{2D}	LONG	vector_overlay     + (lblt_ovl_no << 9)                    'LBLT      Relative, 5(6), 4, -----
{2E}	LONG	vector_overlay     + (lbgt_ovl_no << 9)                    'LBGT      Relative, 5(6), 4, -----
{2F}	LONG	vector_overlay     + (lble_ovl_no << 9)                    'LBLE      Relative, 5(6), 4, -----

{30}	LONG	illegal	                                                   'Illegal
{31}	LONG	illegal	                                                   'Illegal
{32}	LONG	illegal	                                                   'Illegal
{33}	LONG	illegal	                                                   'Illegal
{34}	LONG	illegal	                                                   'Illegal
{35}	LONG	illegal	                                                   'Illegal
{36}	LONG	illegal	                                                   'Illegal
{37}	LONG	illegal	                                                   'Illegal
{38}	LONG	illegal	                                                   'Illegal
{39}	LONG	illegal	                                                   'Illegal
{3A}	LONG	illegal	                                                   'Illegal
{3B}	LONG	illegal	                                                   'Illegal
{3C}	LONG	illegal	                                                   'Illegal
{3D}	LONG	illegal	                                                   'Illegal
{3E}{113F}LONG	vector_overlay 	   + (swi2_ovl_no << 9)                    'SWI3      Inherent,   20, 2, -----
{3F}	LONG	vector_overlay 	   + (swi2_ovl_no << 9)                    'SWI2      Inherent,   20, 2, -----

{40}	LONG	illegal	                                                   'Illegal
{41}	LONG	illegal	                                                   'Illegal
{42}	LONG	illegal	                                                   'Illegal
{43}	LONG	illegal	                                                   'Illegal
{44}	LONG	illegal	                                                   'Illegal
{45}	LONG	illegal	                                                   'Illegal
{46}	LONG	illegal	                                                   'Illegal
{47}	LONG	illegal	                                                   'Illegal
{48}	LONG	illegal	                                                   'Illegal
{49}	LONG	illegal	                                                   'Illegal
{4A}	LONG	illegal	                                                   'Illegal
{4B}	LONG	illegal	                                                   'Illegal
{4C}	LONG	illegal	                                                   'Illegal
{4D}	LONG	illegal	                                                   'Illegal
{4E}	LONG	illegal	                                                   'Illegal
{4F}	LONG	illegal	                                                   'Illegal

{50}	LONG	illegal	                                                   'Illegal
{51}	LONG	illegal	                                                   'Illegal
{52}	LONG	illegal	                                                   'Illegal
{53}	LONG	illegal	                                                   'Illegal
{54}	LONG	illegal	                                                   'Illegal
{55}	LONG	illegal	                                                   'Illegal
{56}	LONG	illegal	                                                   'Illegal
{57}	LONG	illegal	                                                   'Illegal
{58}	LONG	illegal	                                                   'Illegal
{59}	LONG	illegal	                                                   'Illegal
{5A}	LONG	illegal	                                                   'Illegal
{5B}	LONG	illegal	                                                   'Illegal
{5C}	LONG	illegal	                                                   'Illegal
{5D}	LONG	illegal	                                                   'Illegal
{5E}	LONG	illegal	                                                   'Illegal
{5F}	LONG	illegal	                                                   'Illegal

{60}	LONG	illegal	                                                   'Illegal
{61}	LONG	illegal	                                                   'Illegal
{62}	LONG	illegal	                                                   'Illegal
{63}	LONG	illegal	                                                   'Illegal
{64}	LONG	illegal	                                                   'Illegal
{65}	LONG	illegal	                                                   'Illegal
{66}	LONG	illegal	                                                   'Illegal
{67}	LONG	illegal	                                                   'Illegal
{68}	LONG	illegal	                                                   'Illegal
{69}	LONG	illegal	                                                   'Illegal
{6A}	LONG	illegal	                                                   'Illegal
{6B}	LONG	illegal	                                                   'Illegal
{6C}	LONG	illegal	                                                   'Illegal
{6D}	LONG	illegal	                                                   'Illegal
{6E}	LONG	illegal	                                                   'Illegal
{6F}	LONG	illegal	                                                   'Illegal

{70}	LONG	illegal	                                                   'Illegal
{71}	LONG	illegal	                                                   'Illegal
{72}	LONG	illegal	                                                   'Illegal
{73}	LONG	illegal	                                                   'Illegal
{74}	LONG	illegal	                                                   'Illegal
{75}	LONG	illegal	                                                   'Illegal
{76}	LONG	illegal	                                                   'Illegal
{77}	LONG	illegal	                                                   'Illegal
{78}	LONG	illegal	                                                   'Illegal
{79}	LONG	illegal	                                                   'Illegal
{7A}	LONG	illegal	                                                   'Illegal
{7B}	LONG	illegal	                                                   'Illegal
{7C}	LONG	illegal	                                                   'Illegal
{7D}	LONG	illegal	                                                   'Illegal
{7E}	LONG	illegal	                                                   'Illegal
{7F}	LONG	illegal	                                                   'Illegal

{80}	LONG	illegal	                                                   'Illegal
{81}	LONG	illegal	                                                   'Illegal
{82}{1183}LONG	get_word_immediate + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Imediate,    5, 4, -aaaa
{83}	LONG	get_word_immediate + (select_reg_d << 9) + (cmpd << 18)	   'CMPD      Imediate,    5, 4, -aaaa
{84}	LONG	illegal	                                                   'Illegal
{85}	LONG	illegal	                                                   'Illegal
{86}	LONG	illegal	                                                   'Illegal
{87}	LONG	illegal	                                                   'Illegal
{88}	LONG	illegal	                                                   'Illegal
{89}	LONG	illegal	                                                   'Illegal
{8A}	LONG	illegal	                                                   'Illegal
{8B}{118C}LONG	get_word_immediate + (select_reg_s << 9) + (cmpd << 18)    'CMPS      Imediate,    5, 4, -aaaa
{8C}	LONG	get_word_immediate + (select_reg_y << 9) + (cmpd << 18)    'CMPY      Imediate,    5, 4, -aaaa
{8D}	LONG	illegal	                                                   'Illegal
{8E}	LONG	get_word_immediate + (select_reg_y << 9) + (ldd << 18)     'LDY       Imediate,    4, 4, -aa0-
{8F}	LONG	illegal	                                                   'Illegal

{90}	LONG	illegal	                                                   'Illegal
{91}	LONG	illegal	                                                   'Illegal
{92}{1193}LONG	get_word_direct    + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Direct,      7, 3, -aaaa
{93}	LONG	get_word_direct    + (select_reg_d << 9) + (cmpd << 18)    'CMPD      Direct,      7, 3, -aaaa
{94}	LONG	illegal	                                                   'Illegal
{95}	LONG	illegal	                                                   'Illegal
{96}	LONG	illegal	                                                   'Illegal
{97}	LONG	illegal	                                                   'Illegal
{08}	LONG	illegal	                                                   'Illegal
{99}	LONG	illegal	                                                   'Illegal
{9A}	LONG	illegal	                                                   'Illegal
{9B}{119C}LONG	get_word_direct    + (select_reg_s << 9) + (cmpd << 18)	   'CMPS      Direct,      7, 3, -aaaa
{9C}	LONG	get_word_direct    + (select_reg_y << 9) + (cmpd << 18)    'CMPY      Direct,      7, 3, -aaaa
{9D}	LONG	illegal	                                                   'Illegal
{9E}	LONG	get_word_direct    + (select_reg_y << 9) + (ldd << 18)     'LDY       Direct,      6, 3, -aa0-
{9F}	LONG	get_ea_direct      + (select_reg_y << 9) + (std << 18)     'STY       Direct,      6, 3, -aa0-

{A0}	LONG	illegal	                                                   'Illegal
{A1}	LONG	illegal	                                                   'Illegal
{A2}{11A3}LONG	get_word_indexed   + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Indexed,     7, 3, -aaaa
{A3}	LONG	get_word_indexed   + (select_reg_d << 9) + (cmpd << 18)	   'CMPD      Indexed,     7, 3, -aaaa
{A4}	LONG	illegal	                                                   'Illegal
{A5}	LONG	illegal	                                                   'Illegal
{A6}	LONG	illegal	                                                   'Illegal
{A7}	LONG	illegal	                                                   'Illegal
{A8}	LONG	illegal	                                                   'Illegal
{A9}	LONG	illegal	                                                   'Illegal
{AA}	LONG	illegal	                                                   'Illegal
{AB}{11AC}LONG	get_word_indexed   + (select_reg_s << 9) + (cmpd << 18)	   'CMPS      Indexed,     7, 3, -aaaa
{AC}	LONG	get_word_indexed   + (select_reg_y << 9) + (cmpd << 18)	   'CMPY      Indexed,     7, 3, -aaaa
{AD}	LONG	illegal	                                                   'Illegal
{AE}	LONG	get_word_indexed   + (select_reg_y << 9) + (ldd << 18)     'LDY       Indexed,     6, 3, -aa0-
{AF}	LONG	get_ea_indexed     + (select_reg_y << 9) + (std << 18)     'STY       Indexed,     6, 3, -aa0-

{B0}	LONG	illegal	                                                   'Illegal
{B1}	LONG	illegal	                                                   'Illegal
{B2}{11B3}LONG	get_word_extended  + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Extended,    8, 4, -aaaa
{B3}	LONG	get_word_extended  + (select_reg_d << 9) + (cmpd << 18)	   'CMPD      Extended,    8, 4, -aaaa
{B4}	LONG	illegal	                                                   'Illegal
{B5}	LONG	illegal	                                                   'Illegal
{B6}	LONG	illegal	                                                   'Illegal
{B7}	LONG	illegal	                                                   'Illegal
{B8}	LONG	illegal	                                                   'Illegal
{B9}	LONG	illegal	                                                   'Illegal
{BA}	LONG	illegal	                                                   'Illegal
{BB}{11BC}LONG	get_word_extended  + (select_reg_s << 9) + (cmpd << 18)	   'CMPS      Extended,    8, 4, -aaaa
{BC}	LONG	get_word_extended  + (select_reg_y << 9) + (cmpd<< 18)	   'CMPY      Extended,    8, 4, -aaaa
{BD}	LONG	illegal	                                                   'Illegal
{BE}	LONG	get_word_extended  + (select_reg_y << 9) + (ldd << 18)     'LDY       Extended,    7, 4, -aa0-
{BF}	LONG	get_ea_extended	   + (select_reg_y << 9) + (std << 18)     'STY       Extended,    7, 4, -aa0-

{C0}	LONG	illegal	                                                   'Illegal
{C1}	LONG	illegal	                                                   'Illegal
{C2}	LONG	illegal	                                                   'Illegal
{C3}	LONG	illegal	                                                   'Illegal
{C4}	LONG	illegal	                                                   'Illegal
{C5}	LONG	illegal	                                                   'Illegal
{C6}	LONG	illegal	                                                   'Illegal
{C7}	LONG	illegal	                                                   'Illegal
{C8}	LONG	illegal	                                                   'Illegal
{C9}	LONG	illegal	                                                   'Illegal
{CA}	LONG	illegal	                                                   'Illegal
{CB}	LONG	illegal	                                                   'Illegal
{CC}	LONG	illegal	                                                   'Illegal
{CD}	LONG	illegal	                                                   'Illegal
{CE}	LONG	get_word_immediate + (select_reg_s << 9) + (ldd << 18)     'LDS       Imediate,    4, 4, -aa0-
{CF}	LONG	illegal	                                                   'Illegal

{D0}	LONG	illegal	                                                   'Illegal
{D1}	LONG	illegal	                                                   'Illegal
{D2}	LONG	illegal	                                                   'Illegal
{D3}	LONG	illegal	                                                   'Illegal
{D4}	LONG	illegal	                                                   'Illegal
{D5}	LONG	illegal	                                                   'Illegal
{D6}	LONG	illegal	                                                   'Illegal
{D7}	LONG	illegal	                                                   'Illegal
{D8}	LONG	illegal	                                                   'Illegal
{D9}	LONG	illegal	                                                   'Illegal
{DA}	LONG	illegal	                                                   'Illegal
{DB}	LONG	illegal	                                                   'Illegal
{DC}	LONG	illegal	                                                   'Illegal
{DD}	LONG	illegal	                                                   'Illegal
{DE}	LONG	get_word_direct    + (select_reg_s << 9) + (ldd << 18)     'LDS       Direct,      6, 3, -aa0-
{DF}	LONG	get_ea_direct      + (select_reg_s << 9) + (std << 18)     'STS       Direct,      6, 3, -aa0-

{E0}	LONG	illegal	                                                   'Illegal
{E1}	LONG	illegal	                                                   'Illegal
{E2}	LONG	illegal	                                                   'Illegal
{E3}	LONG	illegal	                                                   'Illegal
{E4}	LONG	illegal	                                                   'Illegal
{E5}	LONG	illegal	                                                   'Illegal
{E6}	LONG	illegal	                                                   'Illegal
{E7}	LONG	illegal	                                                   'Illegal
{E8}	LONG	illegal	                                                   'Illegal
{E9}	LONG	illegal	                                                   'Illegal
{EA}	LONG	illegal	                                                   'Illegal
{EB}	LONG	illegal	                                                   'Illegal
{EC}	LONG	illegal	                                                   'Illegal
{ED}	LONG	illegal	                                                   'Illegal
{EE}	LONG	get_word_indexed   + (select_reg_s << 9) + (ldd << 18)     'LDS       Indexed,     6, 3, -aa0-
{EF}	LONG	get_ea_indexed	   + (select_reg_s << 9) + (std << 18)     'STS       Indexed,     6, 3, -aa0-

{F0}	LONG	illegal	                                                   'Illegal
{F1}	LONG	illegal	                                                   'Illegal
{F2}	LONG	illegal	                                                   'Illegal
{F3}	LONG	illegal	                                                   'Illegal
{F4}	LONG	illegal	                                                   'Illegal
{F5}	LONG	illegal	                                                   'Illegal
{F6}	LONG	illegal	                                                   'Illegal
{F7}	LONG	illegal	                                                   'Illegal
{F8}	LONG	illegal	                                                   'Illegal
{F9}	LONG	illegal	                                                   'Illegal
{FA}	LONG	illegal	                                                   'Illegal
{FB}	LONG	illegal	                                                   'Illegal
{FC}	LONG	illegal	                                                   'Illegal
{FD}	LONG	illegal	                                                   'Illegal
{FE}	LONG	get_word_extended  + (select_reg_s << 9) + (ldd << 18)     'LDS       Extended,    7, 4, -aa0-
{FF}	LONG	get_ea_extended    + (select_reg_s << 9) + (std << 18)     'STS       Extended,    7, 4, -aa0-
'---------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------
{
There is no dispatch table for Page 2 instructions.
The following Page 2 ops have been entered into free spaces in the Page 1 dispatch table.
dispatch_table_2
{113F}	LONG	vector_overlay 	   + (swi2_ovl_no << 9)                    'SWI3      Inherent,   20, 2, -----
{1183}	LONG	get_word_immediate + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Imediate,    5, 4, -aaaa
{118C}	LONG	get_word_immediate + (select_reg_s << 9) + (cmpd << 18)    'CMPS      Imediate,    5, 4, -aaaa
{1193}	LONG	get_word_direct    + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Direct,      7, 3, -aaaa
{119C}	LONG	get_word_direct    + (select_reg_s << 9) + (cmpd << 18)	   'CMPS      Direct,      7, 3, -aaaa
{11A3}	LONG	get_word_indexed   + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Indexed,     7, 3, -aaaa
{11AC}	LONG	get_word_indexed   + (select_reg_s << 9) + (cmpd << 18)	   'CMPS      Indexed,     7, 3, -aaaa
{11B3}	LONG	get_word_extended  + (select_reg_u << 9) + (cmpd << 18)	   'CMPU      Extended,    8, 4, -aaaa
{11BC}	LONG	get_word_extended  + (select_reg_s << 9) + (cmpd << 18)	   'CMPS      Extended,    8, 4, -aaaa
}
'---------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------
index_postbyte_table
{00}    LONG  index_post_inc_1   ',R+
{01}    LONG  index_post_inc_2   ',R++
{02}    LONG  index_pre_dec_1    ',-R
{03}    LONG  index_pre_dec_2    ',--R
{04}    LONG  index              ',R
{05}    LONG  index_b            '(+/- B),R
{06}    LONG  index_a            '(+/- A),R
{07}    LONG  illegal            'Illegal
{08}    LONG  index_offset_8     '(+/- 7 bit offset),R
{09}    LONG  index_offset_16    '(+/- 15 bit offset),R
{0A}    LONG  illegal            'Illegal
{0B}    LONG  index_d            '(+/- D),R
{0C}    LONG  pc_offset_8        '(+/- 7 bit offset),PC
{0D}    LONG  pc_offset_16       '(+/- 15 bit offset),PC
{0E}    LONG  illegal            'Illegal
{0F}    LONG  extended_indirect  '[address]
'---------------------------------------------------------------------------------------------------
' The end.
'---------------------------------------------------------------------------------------------------


