'***************************************
'*   mocog_demo v0.6                   *
'*   An emulation of the Motorola 6809 *
'*   (C) 2009 Michael Rychlik          *
'***************************************
'
'v0.0 - 18 May 2009 - Original version
'
'v0.1 - 28 May 2009 - First signs of working, NOP, ADDA $00, JMP $0000 !!!
'
'v0.2 - 15 Jun 2009 - Fixed up print_regs.
'
'v0.3 - 16 Jun 2009 - Included tests for indexed addressing
'
'v0.4 - 21 Jun 2009 - Added memory dump of stack area to print_regs
'                   - Changed included 6809 code to first gcc compiled "Hello world" prog.
'
'v0.5 - 08 Jul 2009 - Implemented RTS, ABX, RTI, RESET, SWI, SWI2, SWI3
'                   - Changed all #defines to upper case.
'
'v0.6 - 14 Jul 2009 - Replaced the old Hello World program with instruction set exerciser ex6809 loaded from file.
'
'This file is a minimal example of how to set up and run the MoCog 6809 emulator
'



CON

'USER CONFIGURATION OPTIONS

'Uncomment one of the following platform types.
'#define TRIBLADEPROP
'#define PROP_DEMO_BOARD

'Uncomment one or more features
'#define BANKED_MEMORY                                  'Uncomment if banked memory is to emulated.
'#define SINGLE_STEP                                    'Uncomment if "single step" emulation required.
'#define HEARTBEAT                                      'Uncomment if emulator is to flash a heartbeat LED
'#define DISPLAY_SECTORS                                'Uncomment to display sectors when starting
'#define DEBUG_FIRST_TIME                               'Uncomment to display the first sector read
#define SHOW_HEX                                        'Uncomment to display in hex, else display char
'#define ENABLE_DBG_CNT                                 'Uncomment to do dbgcnt debug info
#define CONVERT_KEYS                                    'converts F1>CtlZ, F2>CtlS, F3>CtlC
'#define RAM_DISK                                       'use Ram Disk, else SD Card for CPM disk
'#define HDSK_ENABLE                                    'Uncomment for hard disk support

'  baud      = 4800 '115200 too fast for VT100 emulation (also PC_Keyboard & PC_Text)
  baud      = 115200       ''''<===== 4800 '115200 too fast for VT100 emulation (also PC_Keyboard & PC_Text)

#ifdef TRIBLADEPROP
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

' microSD pins on TriBladeProp Blade #2                  
  spiDO     = 9  
  spiClk    = 28 
  spiDI     = 8  
  spiCS     = 14 

  LoadSRAM    = false                                   ' true = loads the SRAM from VB, false skips the load (already done)  
  Disk_Offset = 64*1024                                 ' constant to add to sector no to skip 64KB RAM                       

#else
  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000
#endif

OBJ

#ifdef TRIBLADEPROP
  cpu      : "mocog"                                    'Motorola 6809 emulator in one COG
  kb       : "PC_Keyboard"                              'You can also use Keyboard object for PS2-Keyboard                          
  TV       : "PC_Text"                                  'You can also use TV_Text or VGA_Text objects
  tbp2     : "TriBladeProp_Blade2_202"                  'Blade2 hardware driver 
  debug    : "FullDuplexSerialPlus"                     'reqd to load SRAM from microSD, then closed & kb + TV started
  sd       : "fsrwFemto"                                'SD Software used in FemtoBASIC
#else
  cpu      : "mocog"                                    'Motorola emulator in one COG
  disk     : "sdspiqasm"
  kb       : "PC_Keyboard"                              'You can also use Keyboard object for PS2-Keyboard                          
  TV       : "PC_Text"                                  'You can also use TV_Text or VGA_Text objects
  fcounter : "FrequencyCount"                           'Frequency counter to get execution speed
#endif


CON

'The following taken from SIMH Altair simulator CBIOSX.MAC
ndisks  = 8                     ' total number of Altair disks
tracks  = 254                   ' number of tracks for regular drives
track1  = tracks+1              ' indicator for unknown track position
asecsiz = 137                   ' sector size Altair
csecsiz = $0080                 ' sector size CP/M



'       Address         Mode    Function
'       -------         ----    --------
'       selout          Out     Selects and enables controller and drive
'       statin          In      Indicates status of drive and controller
'       dskcon          Out     Controls disk function
'       secpos          In      Indicates current sector position of disk
'       dskwrit         Out     Write data
'       dskread         In      Read data

selout = 8                      ' port to select and enable controller and drive (OUT)
'       +---+---+---+---+---+---+---+---+
'       | C | X | X | X |   Device      |
'       +---+---+---+---+---+---+---+---+
'
'       C       = If this bit is 1, the disk controller selected by 'device' is
'                 cleared. If the bit is zero, 'device' is selected as the
'                 device being controlled by subsequent I/O operations.
'       X       = not used
'       Device  = value zero thru 15, selects drive to be controlled.

statin = 8                      ' port indicating status of drive and controller (IN)
'       +---+---+---+---+---+---+---+---+
'       | R | Z | I | X | X | H | M | W |
'       +---+---+---+---+---+---+---+---+
'
'       W - When 0, write circuit ready to write another byte.
'       M - When 0, head movement is allowed
'       H - When 0, indicates head is loaded for read/write
'       X - not used (will be 0)
'       I - When 0, indicates interrupts enabled (not used this simulator)
'       Z - When 0, indicates head is on track 0
'       R - When 0, indicates that read circuit has new byte to read

dskcon = 9                      ' port to control disc function (OUT)
'       +---+---+---+---+---+---+---+---+
'       | W | C | D | E | U | H | O | I |
'       +---+---+---+---+---+---+---+---+
'
'       I - When 1, steps head IN one track
'       O - When 1, steps head OUT one track
'       H - When 1, loads head to drive surface
'       U - When 1, unloads head
'       E - Enables interrupts (ignored by this simulator)
'       D - Disables interrupts (ignored by this simulator)
'       C - When 1 lowers head current (ignored by this simulator)
'       W - When 1, starts Write Enable sequence:
'           W bit on device 'statin' (see above) will go 1 and data will be read from
'           port 'dskread' until 137 bytes have been read by the controller from
'           that port. The W bit will go off then, and the sector data will be written
'           to disk. Before you do this, you must have stepped the track to the desired
'           number, and waited until the right sector number is presented on
'           device 'secpos', then set this bit.

secpos = 9                      ' port to indicate current sector position of disk (IN)
'       As the sectors pass by the read head, they are counted and the
'       number of the current one is available in this register.
'
'       +---+---+---+---+---+---+---+---+
'       | X | X |  Sector Number    | T |
'       +---+---+---+---+---+---+---+---+
'
'       X               = Not used
'       Sector number   = binary of the sector number currently under the head, 0-31.
'       T               = Sector True, is a 1 when the sector is positioned to read or write.
'                         Note: MPR according to the CBIOSX this is 0 when sector is positioned.

dskwrit = 10                    ' port to write data (OUT)
dskread = 10                    ' port to read data (IN)

'       All I/O is via programmed I/O. Each device has a status port
'       and a data port. A write to the status port can select
'       some options for the device although the simulator only
'       recognizes the reset command (0x03).
'       A read of the status port gets the port status:
'
'       +---+---+---+---+---+---+---+---+
'       | X | X | X | X | X | X | O | I |
'       +---+---+---+---+---+---+---+---+
'
'       I - A 1 in this bit position means a character has been received
'               on the data port and is ready to be read.
'       O - A 1 in this bit means the port is ready to receive a character
'               on the data port and transmit it out over the serial line.
'
'       A read to the data port gets the buffered character, a write
'       to the data port writes the character to the device.
constat = 16                    ' sio port 1 status port
condata = 17                    ' sio port 1 data port
punstat = 18                    ' sio port 2 status port
pundata = 19                    ' sio port 2 data port

simhport = $FE
srport   = $FF                  ' Altair front panel switch register port

ledport  = $00

' masks for disk controller (statin)
mhm     = $02                   ' head movement mask
mtzero  = $40                   ' head on track zero mask
mall    = $ff                   ' everything ok mask

' commands for disk controller (dskcon)
cstepin = $01                   ' step in command
cstepot = $02                   ' step out command
cload   = $04                   ' load head to drive surface command
cuload  = $08                   ' unload head from drive surface command
cwrseq  = $80                   ' 'start write enable sequence' command

' masks for SIO controller (constat, punstat)
mout    = $02                   ' output allowed mask

' commands for SIO controller (constat, punstat)
creset  = 3                     ' reset command

' constants for hard disk port
hdskNull       = 0                      ' No command
hdskReset      = 1                      ' command to reset controller
hdskRead       = 2                      ' read command
hdskWrite      = 3                      ' write command
hdskParam      = 4                      ' Get HD parameters command
hdskport       = $fd                    ' control port for simulated hard disk

hdsk_base_block = 8 * 254 * 32          ' First block of first hard disk in SD.
                                        ' This is placed after the 8 * 1M Altair floppy areas.(disks * tracks * sectors)

firstSector    = 17                     ' first sector to load
firstTrack     = 0                      ' from this track
firstDiskAddr  = 256*firstTrack+firstSector

'constants for SIMH port
simhreset = 14
stoptimer = 22
setbankselect = 12
hasbankedmemory = 18


'I/O control block commands
io_cmd_out   = $01 
io_cmd_in    = $02
io_cmd_break = $03


'The following taken from the SIMH Altair simulator cfgccp.lib
#ifdef TRIBLADEPROP
  msize   = 64           ' adjust per installed system memory (K bytes)
#else
  msize   = 23           ' adjust per installed system memory (K bytes)
#endif

ram_size = (1024 * msize) - $100 ' Excludes ROM
rom_start = $FF00


'6809 flag bits
entire_flag     = %10000000
fast_int_mask   = %01000000
half_carry_flag = %00100000
interrupt_mask  = %00010000
negative_flag   = %00001000
zero_flag       = %00000100
overflow_flag   = %00000010
carry_flag      = %00000001


DAT

'The 6809 Registers.
'Do not change the order of these.
'Registers are defined in pairs as words so as to enable them to be
'accessed individually as BYTE or in pairs as WORD.
register_file word
d_reg         word      $0000
x_reg         word      $0000
y_reg         word      $0000
u_reg         word      $0000
s_reg         word      $3000
pc_reg        word      $0000
dpcc_reg      word      $0000
  
'IO interface area. Do not change the order of these.
io_cmd_block  long
io_command    long      0
io_port       long      0            
io_data       long      0


'The 6809 will start executing instructions from here
#ifdef TRIBLADEPROP
ram_6809

#else
ram_6809
        file "ex6809.bin"

padding BYTE  0[16 * 1024 - (@padding - @ram_6809) - 2]     'Pad to 16K - 2 bytes
reset_vector BYTE  $00, $00
ram_6809_end

rom_6809
#endif


'Hard Disk Parameter Blocks
'Each DPB is 17 bytes as defined for DPBs in CP/M followed by 2 bytes sector size as used by SIMH
hdsk_dpb
dpb_0                           'Default 8MByte SIMH Altair HDSK params
:spt_low      BYTE      $20     'sectors per track (low byte)
:spt_high     BYTE      $00     'sectors per track (high byte)
:bsh          BYTE      $05     'data allocation Block SHift factor
:blm          BYTE      $1F     'data allocation block mask
:exm          BYTE      $01     'EXtent Mask
:dsm_low      BYTE      $F9     'maximum data block number (low_byte)
:dsm_high     BYTE      $07     'maximum data block number (high_byte)
:drm_low      BYTE      $FF     'total number of directory entries (low byte)
:drm_high     BYTE      $03     'total number of directory entries (high byte)
:al0          BYTE      $FF     'determine reserved directory blocks
:al1          BYTE      $00     'determine reserved directory blocks
:cks_low      BYTE      $00     'size of directory ChecK vector (low byte)
:cks_high     BYTE      $00     'size of directory ChecK vector (high byte)
:off_low      BYTE      $06     'number of reserved tracks (offset) (low byte)
:off_high     BYTE      $00     'number of reserved tracks (offset) (high byte)
:psh          BYTE      $00     'Physical record SHift factor, CP/M 3
:phm          BYTE      $00     'PHhysical record Mask, CP/M 3
:ss_low       BYTE      $80     'Sector Size (low byte)
:ss_high      BYTE      $00     'Sector Size (high byte)
                                'N.B. SS must be 128 for CP/M 2 can be varied for CP/M 3 hard disks.

dpb_1                           'Default 8MByte SIMH Altair HDSK params
:spt_low      BYTE      $20     'sectors per track (low byte)
:spt_high     BYTE      $00     'sectors per track (high byte)
:bsh          BYTE      $05     'data allocation Block SHift factor
:blm          BYTE      $1F     'data allocation block mask
:exm          BYTE      $01     'EXtent Mask
:dsm_low      BYTE      $F9     'maximum data block number (low_byte)
:dsm_high     BYTE      $07     'maximum data block number (high_byte)
:drm_low      BYTE      $FF     'total number of directory entries (low byte)
:drm_high     BYTE      $03     'total number of directory entries (high byte)
:al0          BYTE      $FF     'determine reserved directory blocks
:al1          BYTE      $00     'determine reserved directory blocks
:cks_low      BYTE      $00     'size of directory ChecK vector (low byte)
:cks_high     BYTE      $00     'size of directory ChecK vector (high byte)
:off_low      BYTE      $06     'number of reserved tracks (offset) (low byte)
:off_high     BYTE      $00     'number of reserved tracks (offset) (high byte)
:psh          BYTE      $00     'Physical record SHift factor, CP/M 3
:phm          BYTE      $00     'PHhysical record Mask, CP/M 3
:ss_low       BYTE      $80     'Sector Size (low byte)
:ss_high      BYTE      $00     'Sector Size (high byte)
                                'N.B. SS must be 128 for CP/M 2 can be varied for CP/M 3 hard disks.

dpb_2                           'Default 8MByte SIMH Altair HDSK params
:spt_low      BYTE      $20     'sectors per track (low byte)
:spt_high     BYTE      $00     'sectors per track (high byte)
:bsh          BYTE      $05     'data allocation Block SHift factor
:blm          BYTE      $1F     'data allocation block mask
:exm          BYTE      $01     'EXtent Mask
:dsm_low      BYTE      $F9     'maximum data block number (low_byte)
:dsm_high     BYTE      $07     'maximum data block number (high_byte)
:drm_low      BYTE      $FF     'total number of directory entries (low byte)
:drm_high     BYTE      $03     'total number of directory entries (high byte)
:al0          BYTE      $FF     'determine reserved directory blocks
:al1          BYTE      $00     'determine reserved directory blocks
:cks_low      BYTE      $00     'size of directory ChecK vector (low byte)
:cks_high     BYTE      $00     'size of directory ChecK vector (high byte)
:off_low      BYTE      $06     'number of reserved tracks (offset) (low byte)
:off_high     BYTE      $00     'number of reserved tracks (offset) (high byte)
:psh          BYTE      $00     'Physical record SHift factor, CP/M 3
:phm          BYTE      $00     'PHhysical record Mask, CP/M 3
:ss_low       BYTE      $80     'Sector Size (low byte)
:ss_high      BYTE      $00     'Sector Size (high byte)
                                'N.B. SS must be 128 for CP/M 2 can be varied for CP/M 3 hard disks.

dpb_3                           '720K Super I/O Floppy, for use as RAM disk ?
:spt_low      BYTE      $24     'sectors per track (low byte)
:spt_high     BYTE      $00     'sectors per track (high byte)
:bsh          BYTE      $04     'data allocation Block SHift factor
:blm          BYTE      $0F     'data allocation block mask
:exm          BYTE      $00     'EXtent Mask
:dsm_low      BYTE      $62     'maximum data block number (low_byte)
:dsm_high     BYTE      $01     'maximum data block number (high_byte)
:drm_low      BYTE      $7E     'total number of directory entries (low byte)
:drm_high     BYTE      $00     'total number of directory entries (high byte)
:al0          BYTE      $C0     'determine reserved directory blocks
:al1          BYTE      $00     'determine reserved directory blocks
:cks_low      BYTE      $20     'size of directory ChecK vector (low byte)
:cks_high     BYTE      $00     'size of directory ChecK vector (high byte)
:off_low      BYTE      $02     'number of reserved tracks (offset) (low byte)
:off_high     BYTE      $00     'number of reserved tracks (offset) (high byte)
:psh          BYTE      $02     'Physical record SHift factor, CP/M 3
:phm          BYTE      $03     'PHhysical record Mask, CP/M 3
:ss_low       BYTE      $80     'Sector Size (low byte)
:ss_high      BYTE      $00     'Sector Size (high byte)
                                'N.B. SS must be 128 for CP/M 2 can be varied for CP/M 3 hard disks.


VAR     

'Parameter list for CPU objects, do not change order.
long reg_base       'Address (in HUB)of 6809 register file
long ram_base       'Address (in HUB) of the 6809 RAM area
long ram_len        'Size of 6809 RAM area, assummed to start at zero in 6809 address space.
long rom_base       'Address (in HUB) of the 6809 ROM area
long rom_addr       'Address in 6809 memory space of the start of ROM.
long io_base        'Address (in prop RAM) of the I/O contol interface
long cpu_control_ptr'Quick hack at single step control
long opcode_ptr     'Place where a fetched opcode lives.

word err            'Error code return value
long cpu_control

long frequency      'Frequency counter output goes here

long switch_reg 
long simhcmd

byte altair_drive        'The Altair disk drive currently selected, -1 = none
byte altair_track[16]    'The current track position on each Altair disk drive
byte altair_sector       'The current sector on an altair disk
byte altair_bidx         'Byte index into Altair disk sectors
byte altair_write_enable 'Bytes can be written to the Altair disk when = 1

byte disk_buff[512]
long sd_block

byte key

long dbgcnt
long firsttime
long sector

#ifdef TRIBLADEPROP
'For SRAM CPM Disk Loading
  long  filename
  long  srcname
  byte  buffer[16]
  long  sectorsize
'For microSD routines
  long  ioControl[2]               ' SD parameters                     
  long  drive_base[16]             ' SD base address of drives 0..15 sector 0
  long  blockno                    ' blockno within file
  long  sdsector                   ' sector no 
  long  count                      ' count of chars to read
  byte  buff[8192]                 ' SD buffer
#endif


  byte hdsk_reset_count            'Counts number of consequiteve hdsk reset commands.
  byte hdsk_current_command        'The hdsk command currently in progress
  byte hdsk_last_command           'The last complete hdsk command issued
  byte hdsk_command_pos            'Counts through the outputed command parameter bytes (OUTs)
  byte hdsk_reply_pos              'Counts hdisk port reply bytes (INs)
  long hdsk_dpb_ptr                'Pointer into DPB tables.
  byte hdsk_drive                  'The hdsk drive to read/write with this command
  byte hdsk_sector                 'The hdsk sector to read/write with this command
  word hdsk_track                  'The hdsk track to read/write with this command
  word hdsk_dma                    'The DMA address to read/write from with this command


PUB start

  waitcnt(clkfreq*3 + cnt)      'delay (so you can get PropTerminal active)     <=== 3 secs

#ifdef TRIBLADEPROP
'------------------------------------------------------------------------------
' Load SRAM with 6809 RAM (64KB) & Ram Disk (960KB)
'------------------------------------------------------------------------------
  LoadSRAMfromSD                                        'load from microSD
#endif

'------------------------------------------------------------------------------
' MoCog Emulation starts here
'------------------------------------------------------------------------------
#ifdef TRIBLADEPROP
  reg_base        := @register_file
  ram_base        := $0000                              ' $0000
  ram_len         := ram_size
  rom_base        := $0000                              ' $0000
  rom_addr        := rom_start                  
  io_base         := @io_cmd_block
  cpu_control_ptr := @cpu_control
  switch_reg      := $08                     '$08 is the switch setting SIMH recommends for 4k Basic etc to run     
  firsttime       := 0                       'if 0 display first disk sector read by 6809
#else
  reg_base        := @register_file
  ram_base        := @ram_6809
  ram_len         := ram_size
  rom_base        := @rom_6809
  rom_addr        := rom_start
  io_base         := @io_cmd_block
  cpu_control_ptr := @cpu_control
  switch_reg      := $08                     '$08 is the switch setting SIMH recommends for 4k Basic etc to run     
  firsttime       := 0                       'if 0 display first disk sector read by 6809

  'Load reset vector from highest word in memory
  pc_reg          := (byte[ram_base + $3FFE] << 8) + (byte[ram_base + $3FFF])
#endif

  'Start the frequency counter
#ifdef TRIBLADEPROP
#else
  fcounter.start(@frequency)
#endif

  'Start the terminal
  TV.start(12)
#ifdef TRIBLADEPROP
  TV.str(string("MoCog v0.6 on the TriBladeProp v0.0",13))
#else
  TV.str(string("MoCog v0.6",13))
#endif

  'Start the keyboard
  TV.str (string("Starting keyboard ... "))
  err := \kb.start(26, 27)
  DisplayError(not err)


#ifdef TRIBLADEPROP
#else
  'Start the disk
  TV.str (string("Starting disks (A:B:C:D:E:F:G:H:) ... "))
  err := \disk.start(0)
  DisplayError(err)
#endif

  'Start the TriBladeProp driver
#ifdef TRIBLADEPROP
  TV.str (string("Starting TriBladeProp driver ... "))
  err := tbp2.start
  DisplayError(err)
#endif

#ifdef DISPLAY_SECTORS
'------------------------------------------------------------------------------
' Display any sector
'------------------------------------------------------------------------------
  TV.str(string("Press F1 to display next sector.. '???' to continue..", 13))
  sector := 0
  repeat                          
    repeat until kb.key == $D0
    DisplaySector(sector)
    sector++
#endif

#ifdef TRIBLADEPROP
#else
  TV.str(string("RAM base = "))
  TV.hex(ram_base, 4)
  TV.out(13)

  TV.str(string("ROM base = "))
  TV.hex(rom_base, 4)
  TV.out(13)
#endif

'------------------------------------------------------------------------------
' Start 6809 emulation
'------------------------------------------------------------------------------
  TV.str(string("Starting 6809 emulation..."))
  err := \cpu.start(@reg_base)
  DisplayError(not err)

  'Start the Altair I/O simulation
  service_io

#ifdef TRIBLADEPROP
'------------------------------------------------------------------------------
' TriBladeProp Routines
'------------------------------------------------------------------------------
PRI LoadSRAMfromSD | r, i, j, k
'------------------------------------------------------------------------------
' Load SRAM image from microSD: 6809 64KB + 960KB Ram Disk
'------------------------------------------------------------------------------

  debug.start(31,30,0,baud)     '' Open serial communication with PST           <=== 115200 (was 57600)

'  debug.tx(0)                                           'clear screen
  debug.str(string(13,13))                               'newline
  debug.str(string("TriBladeProp: Load SRAM from microSD (Y/N)? "))
  repeat
    k := debug.rx
    debug.tx(k)                                         'echo
    case k
      "y", "Y" : k := 1
      "n", "N" : k := 2
      other    : k := 0
  while k == 0

  blockno := 0
  count := 8192 
  debug.str (string(13,"Starting disks... ",13))
  FindSDblock                                           'find the address of the SD file block

  if k == 2                                             'check if we are loading sram?
    debug.tx(13)
    waitcnt(clkfreq/100 + cnt)
    debug.stop
    return 0

  debug.str(string(13,"Loading SRAM... "))
  repeat
    ReadSDblock                                         'read from SD
    WriteSRAMblock                                      'write to sram
    blockno++
    debug.tx(".")
  while blockno < 128                                   'eof? (all 1MB)

  debug.str(string(" Loaded.",13))
  waitcnt(clkfreq/4 + cnt)
  debug.stop                                            'stop the serial

PRI FindSDblock | i, n, r                               'read blockno of file for count into buffer

  r := sd.start(@ioControl)                             'start the SD routines
  check_error(not r)
  r := sd.mount(spiDO,spiClk,spiDI,spiCS)               'mount the SD
  check_error(r)

  r := sd.popen(string("DRIVE_1M.RAM"),"r")             'Open file for read (contains sram image)
  check_error(r)
  r := sd.pread(@buff,1)                                'get a char to find the sector address ****
  check_error(r)
  sdsector := ioControl[0] - 1                          'get the sectorno of the file (returns the sector+1)


#ifdef CPM3
'  FindDriveBase(0, string("DRVCPM_3.DSK"))              'set the SD card drive base(s)
  FindDriveBase(0, string("NOBANK3A.DSK"))              'set the SD card drive base(s)    CPM3 nonbanked
  FindDriveBase(1, string("DRIVE__B.DSK"))              'set the SD card drive base(s)
  FindDriveBase(2, string("DRIVE__C.DSK"))              'set the SD card drive base(s)
  FindDriveBase(3, string("DRIVE__D.DSK"))              'set the SD card drive base(s)
  FindDriveBase(4, string("DRIVE__E.DSK"))              'set the SD card drive base(s)
  FindDriveBase(5, string("DRIVE__F.DSK"))              'set the SD card drive base(s)
  FindDriveBase(6, string("DRVCPM_2.DSK"))              'set the SD card drive base(s)   CPM2.2
  FindDriveBase(7, string("DRVCPM_3.DSK"))              'set the SD card drive base(s)   CPM3
'  FindDriveBase(7, string("NONBANK3.DSK"))              'set the SD card drive base(s)
#else
  FindDriveBase(0, string("DRIVE__A.DSK"))              'set the SD card drive base(s)   CPM2.2
  FindDriveBase(1, string("DRIVE__B.DSK"))              'set the SD card drive base(s)
  FindDriveBase(2, string("DRIVE__C.DSK"))              'set the SD card drive base(s)
  FindDriveBase(3, string("DRIVE__D.DSK"))              'set the SD card drive base(s)
  FindDriveBase(4, string("DRIVE__E.DSK"))              'set the SD card drive base(s)
  FindDriveBase(5, string("DRIVE__F.DSK"))              'set the SD card drive base(s)
  FindDriveBase(6, string("DRVCPM_2.DSK"))              'set the SD card drive base(s)   CPM2.2
  FindDriveBase(7, string("DRVCPM_3.DSK"))              'set the SD card drive base(s)   CPM3
#endif
                
  r := sd.stopSDCard                                    'stop SD card, but do not stop cog
  check_error(r)

PRI FindDriveBase(n, st) | r                            'set the SD drive_base for the drive no
  debug.tx($41 + n)
  debug.tx(":")                                         'A:B:C:D:E:F:G:H
  debug.str(st)
  debug.tx(13)
  r := sd.popen(st,"r")                                 'Open file for read (blank image)
  check_error(r)
  r := sd.pread(@buff,1)                                'get a char to find the sector address ****
  check_error(r)
  drive_base[n] := ioControl[0] - 1                     'get the sectorno of the file (returns the sector+1)


PRI ReadSDblock | i, n, r                               'read blockno of file for count into buffer

  r := sd.initSDCard(spiDO,spiClk,spiDI,spiCS)          'init SD card, but do not restart cog
  check_error(r)

' this method relies on the SD file using contiguous sectors !!!
  r := sd.readSDCard(sdsector, @buff, count)            'get a block (ensure buff big enough!)
  check_error(r)
  sdsector := ioControl[0]                              'get the next sectorno

  r := sd.stopSDCard                                    'stop SD card, but do not stop cog
  check_error(r)


PRI WriteSRAMblock | i, r, mask, in

  r := tbp2.start
  check_error(r)

#ifdef DISK_BOOT
  'clears ram $0000-$EFFF, loads $FF00-$FFFF, loads sram disk above
  'remember we are reading 8KB blocks, so clear the seven blocks and 8KB-128B of eighth block
  if blockno < 8
    debug.dec(blockno)
    if blockno == 7
      repeat i from 0 to count-1 -(256)                 'clear below $FF00
        buff[i] := 0                                    'clear the buffer
    else
      repeat i from 0 to count-1
        buff[i] := 0                                    'clear the buffer
'     if blockno == 0
'       buff[0] := $C3                                  'force jmp to $FF00 (faster)
'       buff[1] := $00
'       buff[2] := $FF
#endif

  'write the block out to sram
  r := tbp2.DoCmd("W", @buff, blockno*count, count)    'write to sram
  check_error(r)                
'  show("w")                                  'show part of the block

  r := tbp2.stop
  check_error(r)


PRI preset_137
' Preset the extra 9 bytes (3 before, 6 after) the 128 byte sector
  disk_buff[0] := $A7                           ' add pre & post bytes to make 137 byte sector
  disk_buff[1] := $0E
  disk_buff[2] := $D2
  disk_buff[131] := $B7
  disk_buff[132] := $C8
  disk_buff[133] := $CD
  disk_buff[134] := $A5
  disk_buff[135] := $2B
  disk_buff[136] := $00

PRI show(s) | i, addrx
  addrx := blockno * count
  debug.hex(addrx,6)
  debug.tx(" ")
  repeat i from 0 to 15                                    'only display first xx chars per block
#ifdef SHOW_HEX
    debug.hex(buff[i],2)
    debug.tx(" ")
#else
    debug.tx(buff[i])
#endif
  debug.tx(13)

PRI Check_Error(r)
  if r == -1
    debug.str(string("***Error*** $"))
    debug.hex(r,2)
    sd.unmount
    sd.stop
    debug.str(string(13,"***HALTED***",13))
    repeat                      '<==============  LOOP HERE

PRI CheckError(r)
  if r <> 0 '== -1
    TV.str(string("***Error*** $"))
    TV.hex(r,2)
    sd.stop
    tbp2.stop
    debug.str(string(13,"***HALTED***",13))
    repeat                      '<==============  LOOP HERE

'------------------------------------------------------------------------------
#endif

PRI DisplayError(errx)
  if errx
    TV.str(string(" ***Failed*** $"))
    TV.hex(errx,2)
  else
    TV.str(string(" Passed "))
  TV.str(string(13))

'------------------------------------------------------------------------------
' Service I/O
'-----------------------------------------------------------------------------
PRI service_io | now
'Altair I/O simulation service loop
  now := cnt 
  repeat
    case io_command
      io_cmd_break:                              'CPU break
        on_break    
      io_cmd_in:                                 'An input operation
        on_input
      io_cmd_out:                                'An output operation
        on_output

#ifndef  SINGLE_STEP
    if cnt > now + 80_000_000
      TV.dec (frequency * 2)
      TV.out(13)
      now := cnt
#endif

PRI on_break
'Handle CPU break condition
'No, this never existed in an Altair (or SIMH simulator)
  print_regs
  'waitcnt (cnt + 80_000)
  repeat while kb.key == 0
  io_command := 0


PRI on_input
'Handle IO port reads
  case io_port
    srport:                                      'Altair front panel switches register
      in_srport
    simhport:                                    'SIMH control port
      in_simhport          
    constat:                                     'Console status port.
      in_constat
    condata:                                     'Console data port
      in_condata
    statin:                                      'Altair disk status port
      in_statin
    secpos:                                      'Altair disk sector position port
      in_secpos
    dskread:                                     'Altair disk read port 
      in_dskread
#ifdef HDSK_ENABLE
    hdskport:
      in_hdskport                                'SIMH style hard disk port
#endif
    other:
      'TV.str(string("Read from unimplemented I/O Port $"))
      'TV.hex(io_port,2)
      'TV.out(13)
      'io_data := 0
  io_command := 0                                'Set flag ready for next io          

PRI in_srport
'Handle reads from the Altair front panel switch port
  io_data := switch_reg
  'TV.str(string("SR port read"))
  'TV.out(13)


PRI in_simhport
'Handle reads from the SIMH simulator status port
  if simhcmd == hasbankedmemory
    io_data :=  1
    simhcmd := 0

PRI in_constat
'Handle reads from the console status port
  if kb.gotkey == 0                              'Is there a key press ?
    io_data := %00000010                         'No: Return no char ready for input and output rdy.
  else
    key := kb.key                                'Yes: Get the key
    if key <> 0                                  'NULL character ?
'     TV.out("<")
'     TV.hex(key,4)
'     TV.out(">")
      case key                                   'No: Translate some keys for CP/M
        $00C8: key := $08                        'backspace
        $0263: key := $03                        'control-c
        $0265: key := $05                        'control-e
        $0268: key := $08                        'control-h
        $0270: key := $10                        'control-p
        $0272: key := $12                        'control-r
        $0273: key := $13                        'control-s
        $0275: key := $15                        'control-u
        $0278: key := $18                        'control-x
        $027A: key := $1A                        'control-z
        $00C9: key := $7F                        'rubout
        $00C2: key := $5E                        'up arrow
#ifdef CONVERT_KEYS
        $D0  : key := $1A                        'F1 -> Cltr-Z
        $D1  : key := $13                        'F2 -> Ctrl-S
        $D2  : key := $03                        'F3 -> Ctrl-C
#endif  
          'if key => "a" and key =< "z"          'Use this for 4kBASIC which does not like lower case.
          '  key := key - $20
      io_data := %00000011                       'Return char ready for input and output rdy.
    else
      io_data := %00000010                       'Yes: Return no char ready for input and output rdy.
              
PRI in_condata
'Handle reads from the console data port
          io_data := key                         'Return the keyboard character

PRI in_statin
'Handle reads from the console status port
  io_data := 0
  'if altair_write_enable == 1                   'Ready to accept data ?
    'data |= %00000001
    'data |= %00000010                           'Head movement is always allowed !
    'data |= %00000100                           'Head is loaded for read/write
  if altair_track[altair_drive] <> 0             'Now on track zero ?
    io_data |= %01000000
    'data |= %10000000                           'New byte ready to read

PRI in_secpos
'Handle reads from the floppy disk sector position port
  altair_sector := altair_sector + 1
  if altair_sector > 31
    altair_sector := 0
  io_data := (altair_sector << 1) '| 1           'Return sector and positioned bit
                                                 'Note: MPR BIOS expects positioned bit = 0 when positioned
                                                 'despite what the BIOS comments say about it.
  altair_bidx := 0

PRI in_dskread | bix, r, iy
'Handle reads from the floppy disk read data port
  if altair_bidx == 0
    'Read a new block from SD card

#ifdef TRIBLADEPROP
    preset_137

#ifdef RAM_DISK
    sd_block := (altair_drive * tracks * 32) + (altair_track[altair_drive] * 32) + altair_sector
    err := tbp2.DoCmd("R", @disk_buff[3], sd_block*128 + disk_offset, 128)     'works RR20090403
#else
'----------------------------------------------------------------------------------------------
    r := tbp2.idle                                      'give up the bus
    CheckError(r)
     
    r := sd.initSDcard(spiDO,spiClk,spiDI,spiCS)        'init the SD
    CheckError(r)

    ' this method relies on the SD file using contiguous sectors !!! Only the first 128 bytes of 512 bytes used
    sd_block := (drive_base[altair_drive]) + (altair_track[altair_drive] * 32) + altair_sector
    err := sd.readSDCard(sd_block, @disk_buff[3], 128) 'read a sector
'   CheckError(err)

    r := sd.stopSDcard                                  'stop the SD (free up pins, but keep cog running)
    CheckError(r)
     
    r := tbp2.active                                    'activate the bus
    CheckError(r)
'----------------------------------------------------------------------------------------------
#endif

#else
    sd_block := (altair_drive * tracks * 32) + (altair_track[altair_drive] * 32) + altair_sector
    err := \disk.readblock(sd_block, @disk_buff)
#endif

    if err
      TV.str(string("Disk Read "))
      DisplayError(err)
      DisplayDiskDTS

#ifdef ENABLE_DBG_CNT
    if dbgcnt < 137
      DisplayDiskDTS
#endif

  io_data := disk_buff[altair_bidx]

#ifdef DEBUG_FIRST_TIME
  if firsttime == 0
    firsttime := 1
    TV.str(string(13))
    TV.str(string("Sector="))
    TV.hex(sd_block,8)
    TV.str(string(" "))
    repeat bix from 0 to 136  '256
      TV.hex(disk_buff[bix],2)
      TV.str(string(" "))
    TV.str(string(13))
#endif

#ifdef ENABLE_DBG_CNT
  if dbgcnt++ < 137
    TV.hex(io_data, 2)
    TV.out(" ")
#endif

  if altair_bidx < asecsiz
    altair_bidx++


PRI hdsk_check_dts : status
'Check that selected hdsk drive, sector and track are within limits
  if (hdsk_drive < 1) and (hdsk_track =< 2048) and (hdsk_sector =< 128)
    status := $00
  else
    TV.str(string("Ilegal hdsk access"))
    DisplayHDiskDTS
    status := $FF


PRI in_hdskport_read | r
'Handle read from hdskport during a disk read command
'This read triggers the actual disk sector transfere
'and must return an error code.
'N.B. sectors are mapped to blocks 1 to 1
'  TV.str(string("hdisk read "))
  if hdsk_check_dts                                 'Check drive, track and sector are sane
    io_data := $FF                                  'Return error if not
    return
#ifdef TRIBLADEPROP
'----------------------------------------------------------------------------------------------
  r := tbp2.idle                                        'give up the bus
  CheckError(r)
     
  r := sd.initSDcard(spiDO,spiClk,spiDI,spiCS)          'init the SD
  CheckError(r)

  ' this method relies on the SD file using contiguous sectors !!! Only the first 128 bytes of 512 bytes used
  sd_block := (drive_base[hdsk_drive+16]) + (hdsk_track * 32) + hdsk_sector
  err := sd.readSDCard(sd_block, @disk_buff, 128)       'read a sector
' CheckError(err)

  r := sd.stopSDcard                                    'stop the SD (free up pins, but keep cog running)
  CheckError(r)
     
  r := tbp2.active                                      'activate the bus
  CheckError(r)
'----------------------------------------------------------------------------------------------
#else
  sd_block := hdsk_base_block                       'Start at first block of first HD
  sd_block := sd_block + hdsk_drive * (32 * 2048)   'Skip on to the first block of the selected drive
  sd_block := sd_block + (hdsk_track * 32) + hdsk_sector 'Find correct block for the selected track/sector
  err := \disk.readblock(sd_block, @disk_buff )     'Read the block !
#endif
  if err
    TV.str(string("Error hdisk read "))
    DisplayError(err)
    DisplayHDiskDTS
    io_data := $FF                                  'Return error code
  else
#ifdef TRIBLADEPROP
    r := tbp2.active                                    'activate the bus
    CheckError(r)

    'write the block out to sram
    r := tbp2.DoCmd("W", @disk_buff, hdsk_dma, 128)      'write to sram
    check_error(r)                

    r := tbp2.idle                                      'give up the bus
    CheckError(r)
#else
    BYTEMOVE (@ram_6809 + hdsk_dma, @disk_buff, 128) 'Virtual "DMA" the sector into DMA buffer address in 6809 RAM
#endif
    io_data := $00                                  'Return OK.
  hdsk_last_command := hdskNull                     'Command completed


PRI in_hdskport_write
'Handle read from hdskport during a disk write command
'This read triggers the actual disk sector transfere
'and must return an error code.
'N.B. sectors are mapped to blocks 1 to 1
  if hdsk_check_dts                                 'Check drive, track and sector are sane
    io_data := $FF                                  'Return error if not
    return
#ifdef TRIBLADEPROP
  '???
#else
  sd_block := hdsk_base_block                       'Start at first block of first HD
  sd_block := sd_block + hdsk_drive * (32 * 2048)   'Skip on to the first block of the selected drive
  sd_block := sd_block + (hdsk_track * 32) + hdsk_sector  'Find correct block for the selected track/sector
  BYTEMOVE (@disk_buff, @ram_6809 + hdsk_dma, 128)   'Virtual "DMA" the sector from DMA buffer address in 6809 RAM
  err := \disk.writeblock(sd_block, @disk_buff)     'Write the block!
#endif
  if err
    TV.str(string("Error hdisk write "))
    DisplayError(err)
    DisplayHDiskDTS
    io_data := $FF                                  'Return error code
  else
    io_data := $00                                  'Return OK.
  hdsk_last_command := hdskNull                     'Command completed


PRI in_hdskport_param
'Handle read from hdskport during disk parameters command
'Simply return the next param byte from a DPB.
'  TV.str(string("hdskparam "))
'  TV.dec(hdsk_reply_pos)
'  TV.str(string(" = "))
'  io_data := hdsk_dpb[hdsk_reply_pos++]
  io_data := BYTE[hdsk_dpb_ptr++]
  hdsk_reply_pos++
'  TV.hex(io_data,2)
'      TV.out(13)
  if hdsk_reply_pos == 19
    hdsk_reply_pos := 0
    hdsk_last_command := hdskNull                   'Command completed
'    TV.str(string("command completed "))
'    TV.out(13)


PRI in_hdskport
'Handle reads from the hard disk interface port
'Handling is dispatched to an appropriate function depending on
'the disk operation currently in progress
  case hdsk_last_command
    hdskRead:
      in_hdskport_read
    hdskWrite:
      in_hdskport_write
    hdskParam:                                          'Current command is get disk params so just return successive parameter bytes from DPB.
      in_hdskport_param
    other:
      'TV.str(string("HDSK ERROR 5"))
      'TV.out(13)
      'repeat


PRI on_output
'Handle 6809 IO port writes
  case io_port
    ledport:                                     'An 8 bit parallel output with LEDs attached
      out_ledport          
    simhport:                                    'SIMH control port
      out_simhport
    condata:                                     'Console data port
      out_condata
    constat:                                     'Console status port
      out_constat
    punstat:
      out_punstat                                'Punch control port
    selout:                                      'Altair disk controller select port
      out_selout
    dskcon:                                      'Altair disk function port
      out_dskcon
    dskwrit:                                     'Altair disk write port
      out_dskwrit
#ifdef HDSK_ENABLE
    hdskport:
      out_hdskport                               'SIMH style hard disk port
#endif
    other:
      TV.str(string("Write to unimplemented I/O Port $"))
      TV.hex(io_port,2)
      TV.out(13)
  io_command := 0                                 'Set flag ready for next io

PRI out_ledport
'Handle writes to the LED port
'No, SIMH never had this. I added it for the Propeller demo board.
  'outa[16..23] := io_data                         'Output the port data  (if re-enabled do #ifdef)
  TV.str(string("LED port = "))
  TV.hex(io_data,2)
  TV.out(13)

PRI out_simhport
'Handle writes to the SIMH simulator control port


PRI out_condata
'Handle writes to the console data output port
  'data := data & $7F                             'Use this for 4KBASIC which spits out crap
  if io_data <> 10
    TV.out(io_data)


PRI out_constat
'Handle writes to the console command port
  if io_data == creset
    'TV.str(string("Console reset"))
    'TV.out(13)
  else
    'TV.str(string("Unknown console status write $"))
    'TV.hex(data,2)
    'TV.out(13)


PRI out_punstat
'Handle writes to the punch device command port
  if io_data == creset
    'TV.str(string("Punch reset"))
    'TV.out(13)
  else
    'TV.str(string("Unknown punch status write $"))
    'TV.hex(data,2)
    'TV.out(13)             


PRI out_selout
'Handle writes to the floppy disk drive select port
  if io_data & %10000000
    altair_drive := -1
  else
    altair_drive := io_data & %00001111


PRI out_dskcon
'Handle writes to the floppy disk control port
  if (io_data & cstepin) > 0                     'Step head in command ?
    if altair_track[altair_drive] < tracks
      altair_track[altair_drive]++
  if (io_data & cstepot) > 0                     'Step head out command ?
    if altair_track[altair_drive] > 0
      altair_track[altair_drive]--
          
  if (io_data & cload) > 0                       'Load head to drive surface command ?

  if (io_data & cuload) > 0                      'Unload head from drive surface command ?

  if (io_data & cwrseq) > 0                      ' 'start write enable sequence' command
    altair_bidx := 0
    altair_write_enable := 1


PRI out_dskwrit | r
'Handle writes to the floppy disk write data port
  if altair_bidx < asecsiz
    disk_buff[altair_bidx++] := io_data
    if altair_bidx == asecsiz
      'Write the buffer to the SD card or RamDisk

#ifdef TRIBLADEPROP

#ifdef RAM_DSIK
      sd_block := (altair_drive * tracks * 32) + (altair_track[altair_drive] * 32) + altair_sector
      err := tbp2.DoCmd("W", @disk_buff[3], sd_block*128 + disk_offset, 128)   'works RR20090404
#else
'----------------------------------------------------------------------------------------------
      r := tbp2.idle                                    'give up the bus
      CheckError(r)
       
      r := sd.initSDcard(spiDO,spiClk,spiDI,spiCS)      'init the SD
      CheckError(r)
       
      ' this method relies on the SD file using contiguous sectors !!! Only the first 128 bytes of 512 bytes used
      sd_block := (drive_base[altair_drive]) + (altair_track[altair_drive] * 32) + altair_sector
      err := sd.writeSDCard(sd_block, @disk_buff[3], 128) 'write a sector
'     CheckError(err)
       
      r := sd.stopSDcard                                'stop the SD (free up pins, but keep cog running)
      CheckError(r)
       
      r := tbp2.active                                  'activate the bus
      CheckError(r)
  '----------------------------------------------------------------------------------------------
#endif

#else
      sd_block := (altair_drive * tracks * 32) + (altair_track[altair_drive] * 32) + altair_sector
      err := \disk.writeblock(sd_block, @disk_buff)
#endif
      if err
        TV.str(string("Disk Write "))
        DisplayError(err)
        DisplayDiskDTS

      altair_write_enable := 0


PRI hdsk_get_rw_params
  case hdsk_command_pos
    1:
      hdsk_drive := io_data                    '1st byte is the drive number
      hdsk_command_pos++
    2:
      hdsk_sector := io_data                   '2nd byte is the sector number
      hdsk_command_pos++
    3:
      hdsk_track := io_data                    '3rd byte is LOW of the track number
      hdsk_command_pos++
    4:
      hdsk_track := hdsk_track + (io_data << 8)'4th byte is HIGH of the track number
      hdsk_command_pos++
    5:
      hdsk_dma := io_data                      '5th byte is LOW of the DMA address
      hdsk_command_pos++
    6:
      hdsk_dma := hdsk_dma + (io_data << 8)    '6th (and last) byte is HIGH of the DMA address
      hdsk_command_pos++
      hdsk_last_command := hdsk_current_command 'OUT's for current command completed
      hdsk_current_command := hdskNull
    other:
      TV.str(string("ERROR HDSK 1"))
      TV.out(13)
      repeat


PRI out_hdskport_param
'Handle writes to the hdskport when get params command is in progress
  case hdsk_command_pos
    1:
      hdsk_drive := io_data
'      TV.str(string("Hdsk drive = "))
'      TV.dec(hdsk_drive)
'      TV.out(13)
      hdsk_command_pos++
      hdsk_reply_pos := 0
      hdsk_dpb_ptr := @hdsk_dpb + (hdsk_drive * 19) 'Point to correct DPB for this drive
      hdsk_last_command := hdsk_current_command     'OUT's for current command completed
      hdsk_current_command := hdskNull
    other:
      TV.str(string("HDSK ERROR 4"))
      TV.out(13)
      repeat


PRI out_hdskport_null
'Handle writes to the hdskport when no command is in progress
'io_data should be a new command byte.
  case io_data
    hdskRead:
'      TV.str(string("hdsk read"))
'      TV.out(13)
      hdsk_current_command := hdskRead
      hdsk_command_pos := 1
    hdskWrite:
'      TV.str(string("hdsk write"))
'      TV.out(13)
      hdsk_current_command := hdskWrite
      hdsk_command_pos := 1
    hdskParam:
'      TV.str(string("hdsk param"))
'      TV.out(13)
      hdsk_current_command := hdskParam
      hdsk_command_pos := 1
    hdskReset:
      TV.str(string("hdsk reset"))
      TV.out(13)
      hdsk_current_command := hdskNull
      hdsk_last_command := hdskNull
      hdsk_command_pos := 1
      hdsk_reply_pos := 0
    other:
      TV.str(string("HDSK ERROR 2"))
      TV.out(13)
      repeat



PRI out_hdskport
'Handle writes to the SIMH style hard disk port
'Handling is dispatched to an apropriate function depending
'on the command currently in progress.
'FIXME: RESET should be checked on every write no matter the command in progress.
  case hdsk_current_command
    hdskRead:
      hdsk_get_rw_params
    hdskWrite:
      hdsk_get_rw_params
    hdskParam:
      out_hdskport_param
    hdskReset:
      TV.str(string("HDSK This cannot happen!"))
      TV.out(13)
      repeat
    hdskNull:
      out_hdskport_null
    other:
      TV.str(string("HDSK ERROR 3"))
      TV.out(13)
      repeat

PRI DisplayHDiskDTS
      TV.str(string("D = "))
      TV.hex(hdsk_drive, 2)
      TV.str(string(" T = "))
      TV.hex(hdsk_track, 4)
      TV.str(string(" S = "))
      TV.hex(hdsk_sector, 2)
      TV.str(string(" DMA = "))
      TV.hex(hdsk_dma, 4)
      TV.out(13)

PRI DisplayDiskDTS
      TV.str(string("drive = "))
      TV.hex(altair_drive, 2)
      TV.str(string(" track = "))
      TV.hex(altair_track, 2)
      TV.str(string(" sector = "))
      TV.hex(altair_sector, 2)
      TV.out(13)
          

PRI print_regs | ram_ptr, line, b
'Print the 6809 registers
  TV.str(string("PC="))
  TV.hex(pc_reg, 4)
  TV.out(" ")

  TV.hex(byte[ram_base + pc_reg], 2)
  TV.out(" ")

  TV.str(string("D="))
'  TV.hex((d_reg  >> 8) & $FF, 2)
'  TV.hex(d_reg & $FF, 2)
  TV.hex(d_reg, 4)
  TV.out(" ")
      
  TV.str(string("X="))
  TV.hex(x_reg, 4)
  TV.out(" ")
      
  TV.str(string("Y="))
  TV.hex(y_reg, 4)
  TV.out(" ")

  TV.str(string("U="))
  TV.hex(u_reg, 4)
  TV.out(" ")
      
  TV.str(string("S="))
  TV.hex(s_reg, 4)
  TV.out(" ")

  TV.str(string("DP="))
  TV.hex((dpcc_reg >> 8) & $FF, 2)
  TV.out(" ")

  TV.str(string("CC="))
  TV.hex(dpcc_reg & $FF, 2)
  TV.out(13)

  'TV.str(string("testbyte="))
  'TV.hex(byte[ram_base + $0400], 2)
  'TV.out(13)

  'Display 32 bytes of the stack area
  ram_ptr := ram_base + $400 - $20
  repeat line from 0 to 1
    TV.hex(ram_ptr - ram_base, 4)
    TV.out(" ")
    repeat b from 0 to 15
      TV.hex(byte[ram_ptr++], 2)
      TV.out(" ")
    TV.out(13)




#ifdef DISPLAY_SECTORS
PRI DisplaySector(s) | ix
'Read and display a 128B sector (SRAM, no offset)
  err := tbp2.DoCmd("R", @disk_buff, s*128, 128)
  TV.str(string("Sector="))
  TV.hex(s,8)
  TV.str(string(" "))
  DisplayError(err)
  repeat ix from 0 to 127
    TV.hex(disk_buff[ix],2)
    TV.str(string(" "))
  TV.str(string(13))
#endif
    
